;################################################################################
;#										#
;# libi2c - i2c library for ATmega controllers and 24C16			#
;# copyright (c) 2007-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 2		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; withsts even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.if ctype==8
;-------------------------------------------------------------------------------
; initialize twi and send stop
;-------------------------------------------------------------------------------
libi2c_initl:	ldi	tempreg1,92		;set bitrate
		out	TWBR,tempreg1
		ldi	tempreg1,(1<<TWEN) | (1<<TWINT)	;enable twi
		out	TWCR,tempreg1

;-------------------------------------------------------------------------------
; generate stop condition
;-------------------------------------------------------------------------------
libi2c_stop:	ldi	tempreg1,(1<<TWSTO) | (1<<TWINT) | (1<<TWEN)
		out	TWCR,tempreg1			;set control
		ret

;-------------------------------------------------------------------------------
; generate start condition
;-------------------------------------------------------------------------------
libi2c_start:	ldi	tempreg1,(1<<TWSTA) | (1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; write byte (tempreg2) to I2C
;-------------------------------------------------------------------------------
libi2c_wbyte:	out	TWDR,tempreg2
		ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (ACK)
;-------------------------------------------------------------------------------
libi2c_rbyte:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN) | (1<<TWEA)
		rcall	libi2c_wready
		in	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (NACK)
;-------------------------------------------------------------------------------
libi2c_rbyten:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rcall	libi2c_wready
		in	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; set control to tempreg1 and wait for TWI is ready
;-------------------------------------------------------------------------------
libi2c_wready:	out	TWCR,tempreg1			;set control
libi2c_wready1:	in	tempreg1,TWCR			;get control
		sbrs	tempreg1,TWINT			;skip if ready
		rjmp	libi2c_wready1			;loop
		in	tempreg3,TWSR			;get status
		andi	tempreg3,0xf8			;mask bits
		ret					;thats all
.endif

.if ctype==16
;-------------------------------------------------------------------------------
; initialize twi and send stop
;-------------------------------------------------------------------------------
libi2c_initl:	ldi	tempreg1,92		;set bitrate
		out	TWBR,tempreg1
		ldi	tempreg1,(1<<TWEN) | (1<<TWINT)	;enable twi
		out	TWCR,tempreg1

;-------------------------------------------------------------------------------
; generate stop condition
;-------------------------------------------------------------------------------
libi2c_stop:	ldi	tempreg1,(1<<TWSTO) | (1<<TWINT) | (1<<TWEN)
		out	TWCR,tempreg1			;set control
		ret

;-------------------------------------------------------------------------------
; generate start condition
;-------------------------------------------------------------------------------
libi2c_start:	ldi	tempreg1,(1<<TWSTA) | (1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready
		
;-------------------------------------------------------------------------------
; write byte (tempreg2) to I2C
;-------------------------------------------------------------------------------
libi2c_wbyte:	out	TWDR,tempreg2
		ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (ACK)
;-------------------------------------------------------------------------------
libi2c_rbyte:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN) | (1<<TWEA)
		rcall	libi2c_wready
		in	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (NACK)
;-------------------------------------------------------------------------------
libi2c_rbyten:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rcall	libi2c_wready
		in	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; set control to tempreg1 and wait for TWI is ready
;-------------------------------------------------------------------------------
libi2c_wready:	out	TWCR,tempreg1			;set control
libi2c_wready1:	in	tempreg1,TWCR			;get control
		sbrs	tempreg1,TWINT			;skip if ready
		rjmp	libi2c_wready1			;loop
		in	tempreg3,TWSR			;get status
		andi	tempreg3,0xf8			;mask bits
		ret					;thats all
.endif

.if ctype==88
;-------------------------------------------------------------------------------
; initialize twi and send stop
;-------------------------------------------------------------------------------
libi2c_initl:	ldi	tempreg1,92		;set bitrate
		sts	TWBR,tempreg1
		ldi	tempreg1,(1<<TWEN) | (1<<TWINT)	;enable twi
		sts	TWCR,tempreg1

;-------------------------------------------------------------------------------
; generate stop condition
;-------------------------------------------------------------------------------
libi2c_stop:	ldi	tempreg1,(1<<TWSTO) | (1<<TWINT) | (1<<TWEN)
		sts	TWCR,tempreg1			;set control
		ret

;-------------------------------------------------------------------------------
; generate start condition
;-------------------------------------------------------------------------------
libi2c_start:	ldi	tempreg1,(1<<TWSTA) | (1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; write byte (tempreg2) to I2C
;-------------------------------------------------------------------------------
libi2c_wbyte:	sts	TWDR,tempreg2
		ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rjmp	libi2c_wready

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (ACK)
;-------------------------------------------------------------------------------
libi2c_rbyte:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN) | (1<<TWEA)
		rcall	libi2c_wready
		lds	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; read byte (tempreg2) from I2C (NACK)
;-------------------------------------------------------------------------------
libi2c_rbyten:	ldi	tempreg1,(1<<TWINT) | (1<<TWEN)
		rcall	libi2c_wready
		lds	tempreg2,TWDR			;get data
		ret

;-------------------------------------------------------------------------------
; set control to tempreg1 and wait for TWI is ready
;-------------------------------------------------------------------------------
libi2c_wready:	sts	TWCR,tempreg1			;set control
		nop
libi2c_wready1:	lds	tempreg1,TWCR			;get control
		sbrs	tempreg1,TWINT			;skip if ready
		rjmp	libi2c_wready1			;loop
		lds	tempreg3,TWSR			;get status
		andi	tempreg3,0xf8			;mask bits
		ret					;thats all
.endif

;-------------------------------------------------------------------------------
; calc first byte
;-------------------------------------------------------------------------------
libi2c_cfb:	mov	tempreg2,XH		;copy high address
		lsl	tempreg2
		andi	tempreg2,0x0e		;mask
		ori	tempreg2,0xa0		;EEPROM devsel
		ret

;-------------------------------------------------------------------------------
; write memory address X to I2C EEPROM
;-------------------------------------------------------------------------------
libi2c_wad:	rcall	libi2c_start		;start condition
		cpi	tempreg3,0x08
		brne	libi2c_wad_e		;error
		rcall	libi2c_cfb
		rcall	libi2c_wbyte
		cpi	tempreg3,0x18
		brne	libi2c_wad_e		;error
		mov	tempreg2,XL		;write address 
		rcall	libi2c_wbyte		
		clr	ereg
		ret
libi2c_wad_e:	ldi	ereg,1			;set error
		rjmp	libi2c_stop		;stop condition and end

;-------------------------------------------------------------------------------
; set EEPROM to read
;-------------------------------------------------------------------------------
libi2c_srd:	rcall	libi2c_start		;start condition
		rcall	libi2c_cfb
		ori	tempreg2,0x01		;set rw
		rcall	libi2c_wbyte		
		ret

;-------------------------------------------------------------------------------
; read data byte from X to tempreg2 
;-------------------------------------------------------------------------------
.ifdef use_libi2c_read
libi2c_read:	push	tempreg1
		push	tempreg3
		rcall	libi2c_wad		;write address
		cpi	ereg,0			;error?
		brne	libi2c_read_2		;yes
		rcall	libi2c_srd		;set to read
libi2c_read_1:	rcall	libi2c_rbyten		;read byte with NACK
libi2c_read_2:	rcall	libi2c_stop
		adiw	XL,1
		pop	tempreg3
		pop	tempreg1
		ret
.endif

;-------------------------------------------------------------------------------
; write data byte tempreg2 to external eeprom address X
;-------------------------------------------------------------------------------
.ifdef use_libi2c_write
libi2c_write:	push	tempreg1
		push	tempreg3
		push	tempreg2
		rcall	libi2c_wad		;write address
		pop	tempreg2
		cpi	ereg,0			;error?
		brne	libi2c_write_2
libi2c_write_1:	rcall	libi2c_wbyte		;write byte
libi2c_write_2:	rjmp	libi2c_read_2
.endif
