;################################################################################
;#										#
;# libmio - multi i/o for ATMega 8						#
;# copyright (c) 2008-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;# This library is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public				#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;-------------------------------------------------------------------------------
;video-interrupt (no registers destroyed, stack=13)
;save registers
;-------------------------------------------------------------------------------
libmio_isr:	push	ZH			;2 save Z-register
		push	ZL			;2 save Z-register
		push	YH			;2 save Y-register
		push	YL			;2 save Y-register
		push	XH			;2 save X-register
		push	XL			;2 save X-register
		push	r0			;2 save r0
		push	r1			;2 save r1
		in	ZH,SREG			;1 save status
		push	ZH			;2
		ldi	YL,LOW(libmio_ram)	;1 set ptr
		ldi	YH,HIGH(libmio_ram)	;1

;-------------------------------------------------------------------------------
;serial port io
;-------------------------------------------------------------------------------
		ldd	XH,Y+libmio_serout	;2 serial io
		sbrs	XH,7			;1 skip if 1-bit
		cbi	seroport,seropin	;1 clear pin
		sbrc	XH,7			;1 skip if 0-bit
		sbi	seroport,seropin	;1 set pin
		in	XH,seriport		;1
.if std_uart == 0
		com	XH			;invert signal if necessary
		sbrc	vline_l,0		;charge pump
		sbi	sercport,sercpin
		sbrs	vline_l,0
		cbi	sercport,sercpin
.endif
libmio_isrser:
		std	Y+libmio_serin,XH	;2

;-------------------------------------------------------------------------------
;count line up
;-------------------------------------------------------------------------------
		add	vline_l,const_1		;line +1
		adc	vline_h,const_0
		ldd	XL,Y+libmio_pwm
.if ctype == mega8
		out	OCR2,XL
.endif
.if ctype == mega16
		out	OCR2,XL
.endif
.if ctype == mega88
		sts	OCR2A,XL
.endif

;-------------------------------------------------------------------------------
;check, if line is image, empty or vsync
;-------------------------------------------------------------------------------
		ldi	XL,libmio_pixel_rows	;visible lines+1
		cp	vline_l,XL		;number of visible lines
		sbrs	vline_h,0		;1 skip if no image
		brcs	libmio_iss		;2 goto image-line (short jump)

libmio_is_08:	ldi	XL,libmio_vstart	;vsync start
		cp	vline_l,XL
		brne	libmio_is_09
.if ctype == mega8
		ldi	XH,0x82			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
.endif
.if ctype == mega16
		ldi	XH,0x82			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
.endif
.if ctype == mega88
		ldi	XH,0x82			;FAST-PWM-Mode 14 (PWM channel B)
		sts	TCCR1A,XH
.endif
		rjmp	libmio_is_15		;2

libmio_is_09:	ldi	XL,libmio_vstop		;1 vstop
		cp	vline_l,XL
		brne	libmio_is_10

.if ctype == mega8
		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
.endif
.if ctype == mega16
		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
.endif
.if ctype == mega88
		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		sts	TCCR1A,XH
.endif
		rjmp	libmio_is_15		;2 vsync line is empty 

libmio_is_10:	ldi	XL,libmio_maxlines	;1 maxlines (7/57)
		cp	vline_l,XL
		brne	libmio_is_12		;2 end of frame

libmio_is_11:	ldi	XL,LOW(libmio_vram)	;1 set vram start
		ldi	XH,HIGH(libmio_vram)	;1
		rcall	libmio_stx1		;1
		mov	vline_l,const_0		;1 line_l=0
		mov	vline_h,const_0		;1 line_h=0
libmio_env:	ldd	XL,Y+libmio_envval
		cpi	XL,0x00
		breq	libmio_env_e
		mov	XH,XL
		lsr	XH
		lsr	XH
		inc	XH
		sub	XL,XH
		std	Y+libmio_envval,XL
libmio_env_e:
		ldd	XL,Y+libmio_frames
		inc	XL
		cpi	XL,50
		brne	libmio_is_11a
		clr	XL
libmio_is_11a:	std	Y+libmio_frames,XL
libmio_is_12:	rjmp	libmio_is_15

libmio_stx1:	std	Y+libmio_rampos_l,XL	;2
		std	Y+libmio_rampos_h,XH	;2
		std	Y+libmio_cline,const_0	;2 cline=0
		ret

;-------------------------------------------------------------------------------
;sync to counter
;-------------------------------------------------------------------------------
libmio_iss:
.if ctype == mega8
		in	XL,TCNT1L		;1 get timer
.endif
.if ctype == mega16
		in	XL,TCNT1L		;1 get timer
.endif
.if ctype == mega88
		lds	XL,TCNT1L		;1 get timer
.endif
		cpi	XL,libmio_hstart+59 	;1
		brcs	libmio_iss		;1 loop
libmio_iss_01:	cpi	XL,libmio_hstart+60
		brcs	libmio_iss_02
libmio_iss_02:	cpi	XL,libmio_hstart+61
		brcs	libmio_iss_03

.if ctype == mega8
libmio_iss_03:	cpi	XL,libmio_hstart+62
		brcs	libmio_iss_05
.endif
.if ctype == mega16
libmio_iss_03:	cpi	XL,libmio_hstart+62
		brcs	libmio_iss_05
.endif
.if ctype == mega88
libmio_iss_03:	cpi	XL,libmio_hstart+62
		brcs	libmio_iss_04
libmio_iss_04:	cpi	XL,libmio_hstart+63
		brcs	libmio_iss_05
.endif

libmio_iss_05:	ldd	XL,Y+libmio_fastmode
		sbrc	XL,0
		rjmp	libmio_is_15

;-------------------------------------------------------------------------------
; -output character line
;-------------------------------------------------------------------------------
.include	"libmio/vgen.asm"

;------------------------------------------------------------------------------
;the keyboard routine
;------------------------------------------------------------------------------
libmio_kir:
		ldd	XL,Y+libmio_kbdstate	;get key state
		sbrc	XL,5			;2nd cycle flag
		rjmp	libmio_kbd_06a		;jump to keyboard cycle 2
.if ctype == mega8
	in	ZL,UCSRA		;status
.endif
.if ctype == mega16
	in	ZL,UCSRA		;status
.endif
.if ctype == mega88
	lds	ZL,UCSR0A		;status
.endif
		sbrs	ZL,7			;RXC?
		rjmp	libmio_kbd_12		;no byte available
.if ctype == mega8
		in	ZL,UDR
.endif
.if ctype == mega16
		in	ZL,UDR
.endif
.if ctype == mega88
		lds	ZL,UDR0
.endif
		cpi	ZL,0xf0			;key release
		brne	libmio_kbd_01		;jump if not
		ori	XL,0x80			;set release flag
		rjmp	libmio_kbd_11		;go to the end

libmio_kbd_01:	cpi	ZL,0xe0			;extended code
		brne	libmio_kbd_01a		;jump if not
		ori	XL,0x40			;set extended flag
		rjmp	libmio_kbd_11		;thats all

libmio_kbd_01a:	cpi	ZL,0x83			;extended code
		brne	libmio_kbd_02		;jump if not
		ldi	ZL,0x10

libmio_kbd_02:	sbrc	ZL,7			;skip if no bit 7
		rjmp	libmio_kbd_12		;thats all

		cpi	ZL,0x12			;left shift
		brne	libmio_kbd_03		;no
		ori	XL,0x01			;set shift bit
		sbrc	XL,7			;skip if not release
		andi	XL,0xfe			;clear shift bit
		rjmp	libmio_kbd_10		;thats all
		
libmio_kbd_03:	cpi	ZL,0x59			;right shift
		brne	libmio_kbd_04		;jump if not
		ori	XL,0x02			;set shift bit
		sbrc	XL,7			;skip if not release
		andi	XL,0xfd			;clear shift bit
		rjmp	libmio_kbd_10		;thats all

libmio_kbd_04:	cpi	ZL,0x14			;control
		brne	libmio_kbd_05		;jump if not
		sbrc	XL,6			;extended? (right ctrl key)
		rjmp	libmio_kbd_04a
		ori	XL,0x04			;set control bit
		sbrc	XL,7			;skip if not release
		andi	XL,0xfb			;clear ctrl bit
		rjmp	libmio_kbd_10		;thats all
libmio_kbd_04a:	ori	XL,0x08			;set control bit
		sbrc	XL,7			;skip if not release
		andi	XL,0xf7			;clear ctrl bit
		rjmp	libmio_kbd_10		;thats all

libmio_kbd_05:	cpi	ZL,0x11			;alternate
		brne	libmio_kbd_06		;jump if not
		ori	XL,0x10			;set alt bit
		sbrc	XL,7			;skip if not release
		andi	XL,0xef			;clear alt bit
		rjmp	libmio_kbd_10		;thats all

libmio_kbd_06:	sbrc	XL,7			;check release flag
		rjmp	libmio_kbd_06e		;RELEASED
		ori	XL,0x20			;set bit for cycle 2
		std	Y+libmio_scancode,ZL	;store scancode
		rjmp	libmio_kbd_11		;jump to end

libmio_kbd_06e:	std	Y+libmio_keycode,const_0	;clear keycode
		rjmp	libmio_kbd_10		;jump to end

libmio_kbd_06a:	ldd	ZL,Y+libmio_scancode	;get scancode
		andi	XL,0xdf			;kill cycle 2 flag
		bst	XL,6			;check extended flag
		brtc	libmio_kbd_07		;branch, if not extended
		cpi	ZL,0x68			;cursor?
		brcs	libmio_kbd_07		;branch, if not
		mov	XH,ZL			;
		subi	XH,0x68			; 
		ldi	ZH,HIGH(libmio_ktab2*2)	;start keytable 2
		ldi	ZL,LOW(libmio_ktab2*2)	;start keytable 2
		rjmp	libmio_kbd_08		;

libmio_kbd_07:	mov	XH,ZL			;char
		sbrc	XL,0			;left shift active
		ori	XH,0x80			;part 2 of table
		sbrc	XL,1			;right shift active
		ori	XH,0x80			;part 2 of table
		ldi	ZH,HIGH(libmio_ktab1*2)	;start keytable 1
		ldi	ZL,LOW(libmio_ktab1*2)	;start keytable 1
libmio_kbd_08:	add	ZL,XH			;
		adc	ZH,const_0		;
		lpm	ZL,Z			;get value
libmio_kbd_8a:	std	Y+libmio_keycode,ZL	;keycode
		std	Y+libmio_lastkey,ZL	;lastkey
		sbrs	XL,2			;control?
		rjmp	libmio_kbd_10		;no
		cpi	ZL,0xe9			;del?
		brne	libmio_kbd_09
		sbrs	XL,4			;alternate?
		rjmp	libmio_kbd_09
		jmp	start			;restart
libmio_kbd_09:	ldd	XH,Y+libmio_kflags	;keyboard control
		cpi	ZL,'c'			;C
		brne	libmio_kbd_09a
		ori	XH,1			;set break bit
		rjmp	libmio_kbd_09c

libmio_kbd_09a:	cpi	ZL,'p'			;C
		brne	libmio_kbd_09c
		ori	XH,4			;set screenshot bit

libmio_kbd_09c:	std	Y+libmio_kflags,XH	;save keyboard control
libmio_kbd_10: 	andi	XL,0x3f			;clear ext and release bit
libmio_kbd_11:	std	Y+libmio_kbdstate,XL	;store keyboard status
libmio_kbd_12:

;-------------------------------------------------------------------------------
;calculate sound
;-------------------------------------------------------------------------------
libmio_snd:	ldd	r0,Y+libmio_dds_l
		ldd	r1,Y+libmio_dds_h
		ldi	ZL,LOW(ddstab*2)
		ldi	ZH,HIGH(ddstab*2)
		ldd	XL,Y+libmio_tone
		ldd	XH,Y+libmio_rand3
		andi	XL,0x3f
		cpi	XL,0x3f
		breq	libmio_snd_1
		lsl	XL
		add	ZL,XL
		lpm	XL,Z+
		lpm	XH,Z+
		add	XL,r0
		adc	XH,r1
		std	Y+libmio_dds_l,XL
		std	Y+libmio_dds_h,XH
libmio_snd_1:	ldd	XL,Y+libmio_envval
		lsr	XH
		mul	XL,XH
		ldi	XL,0x80
		sub	XL,r1
		std	Y+libmio_pwm,XL
libmio_snd_e:

;-------------------------------------------------------------------------------
;serial port (receive)
;-------------------------------------------------------------------------------
libmio_ser_00:	ldd	XL,Y+libmio_rbcnt	;bit counter
		cpi	XL,0x00			;active
		breq	libmio_ser_08		;no
		ldd	ZH,Y+libmio_serin	;get in data
		cpi	XL,0x0a			;wait for start bit
		brne	libmio_ser_01		;no
		sbrc	ZH,seripin		;skip if line is 0
		rjmp	libmio_ser_08		;we must wait again
		ldi	XH,19			;1 1/2 bit times
		rjmp	libmio_ser_05		;thats all

libmio_ser_01:	ldd	XH,Y+libmio_rcnt	;clock counter
		dec	XH			;-1
		brne	libmio_ser_07		;
		cpi	XL,1			;only stopp bit
		brne	libmio_ser_02		;
		sbrc	ZH,seripin		;skip if line is 0
		rjmp	libmio_ser_05		;all done
		ldi	XH,1
		rjmp	libmio_ser_06		;we must wait

libmio_ser_02:	ldd	r0,Y+libmio_rdat	;serial data
		lsr	r0
		bst	ZH,seripin		;RXD data
		bld	r0,7			;shift in
		std	Y+libmio_rdat,r0

libmio_ser_04:	ldi	XH,13
libmio_ser_05:	dec	XL
libmio_ser_06:	std	Y+libmio_rbcnt,XL
libmio_ser_07:	std	Y+libmio_rcnt,XH
libmio_ser_08:


;-------------------------------------------------------------------------------
;serial port (send)
;-------------------------------------------------------------------------------
libmio_set_00:	ldd	XL,Y+libmio_tbcnt	;bit counter
		cpi	XL,0x00			;check if active
		breq	libmio_set_08		;nothing to do
		cpi	XL,0x0b			;start bit?
		brne	libmio_set_01
		clr	ZH			;start-bit is 0
		rjmp	libmio_set_04

libmio_set_01:	ldd	XH,Y+libmio_tcnt	;clock counter
		inc	XH			;+1
		cpi	XH,13			;13 cycles per bit
		brne	libmio_set_07		;not reached
		ldi	ZH,0xff			;stop bit
		cpi	XL,3			;max bits
		brcs	libmio_set_04		;only stop remained

libmio_set_03:	ldd	r0,Y+libmio_tdat
		lsr	r0
		ror	ZH
		std	Y+libmio_tdat,r0
libmio_set_04:	std	Y+libmio_serout,ZH	;set bit value
libmio_set_05:	dec	XL			;bitcount -
		clr	XH			;clear cycle counter
libmio_set_06:	std	Y+libmio_tbcnt,XL
libmio_set_07:	std	Y+libmio_tcnt,XH
libmio_set_08:

;-------------------------------------------------------------------------------
;restore registers and exit
;-------------------------------------------------------------------------------
libmio_is_20:	rcall	libmio_rand
		pop	ZH			;2 restore sreg
		out	SREG,ZH			;1 put back
		pop	r1			;2 restore r1
		pop	r0			;2 restore r0
		pop	XL			;2 restore X-register
		pop	XH			;2 restore X-register
libmio_is_30:	pop	YL			;2 restore Y-register
		pop	YH			;2 restore Y-register
		pop	ZL			;2 restore Z-register
		pop	ZH			;2 restore Z-register
		reti				;4 thats all

;-------------------------------------------------------------------------------
;port konfiguration for port B ... D
;-------------------------------------------------------------------------------
libmio_init:
.if ctype == mega8
		ldi	XL,0x3e			;port direction
		out	DDRB,XL
		ldi	XL,0x30			;port direction
		out	DDRC,XL
		ldi	XL,0x80			;set pin 4-7 as output
		out	DDRD,XL

		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
		ldi	XH,0x19			;FAST-PWM-Mode (no prescaler)
		out	TCCR1B,XH
		ldi	ZH,HIGH(libmio_counter-1)
		out	ICR1H,ZH
		ldi	ZH,LOW(libmio_counter-1)
		out	ICR1L,ZH

		out	OCR1AH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hslen		;1 hsync length 
		out	OCR1AL,ZH		

		out	OCR1BH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hstart	;1 hor position
		out	OCR1BL,ZH		

		ldi	XL,0x08
		out	TIMSK,XL		;enable compare OC1B int

		ldi	XH,0x69			;FAST-PWM-Mode
		out	TCCR2,XH
		ldi	XH,0x80			;neutral
		out	OCR2,XH
.endif

.if ctype == mega16
		ldi	XL,0x0a			;port direction
		out	DDRB,XL
		ldi	XL,0xff
		out	PORTB,XL
		ldi	XL,0x80			;port direction
		out	DDRC,XL
		ldi	XL,0xf0			;set pin 4-7 as output
		out	DDRD,XL

		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		out	TCCR1A,XH
		ldi	XH,0x19			;FAST-PWM-Mode (no prescaler)
		out	TCCR1B,XH
		ldi	ZH,HIGH(libmio_counter-1)
		out	ICR1H,ZH
		ldi	ZH,LOW(libmio_counter-1)
		out	ICR1L,ZH

		out	OCR1AH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hslen		;1 hsync length 
		out	OCR1AL,ZH

		out	OCR1BH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hstart	;1 hor position
		out	OCR1BL,ZH

		ldi	XL,0x08
		out	TIMSK,XL		;enable compare OC1B int

		ldi	XH,0x69			;FAST-PWM-Mode
		out	TCCR2,XH
		ldi	XH,0x80			;neutral
		out	OCR2,XH
.endif

.if ctype == mega88
		ldi	XL,0x3e			;port direction
		out	DDRB,XL
		ldi	XL,0x30			;port direction
		out	DDRC,XL
		ldi	XL,0x80			;set pin 4-7 as output
		out	DDRD,XL

		ldi	XH,0xc2			;FAST-PWM-Mode 14 (PWM channel B)
		sts	TCCR1A,XH
		ldi	XH,0x19			;FAST-PWM-Mode (no prescaler)
		sts	TCCR1B,XH
		ldi	ZH,HIGH(libmio_counter-1)
		sts	ICR1H,ZH
		ldi	ZH,LOW(libmio_counter-1)
		sts	ICR1L,ZH

		sts	OCR1BH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hslen		;1 hsync length 
		sts	OCR1BL,ZH		

		sts	OCR1AH,const_0		;we use only 8 bits
		ldi	ZH,libmio_hstart	;1 hor position
		sts	OCR1AL,ZH

		ldi	XL,0x02
		sts	TIMSK1,XL		;enable compare OC1B int

		ldi	XH,0xc3			;FAST-PWM-Mode
		sts	TCCR2A,XH
		ldi	XH,0x01			;no prescaler
		sts	TCCR2B,XH
		ldi	XH,0x80			;neutral
		sts	OCR2A,XH
.endif

;-------------------------------------------------------------------------------
;usart configuration (synchron for keyboard)
;-------------------------------------------------------------------------------
.if ctype == mega8
		out	UCSRA,const_0
		ldi	XL,0x10
		out	UCSRB,XL
		ldi	XL,0xe6
		out	UCSRC,XL
.endif

.if ctype == mega16
		out	UCSRA,const_0
		ldi	XL,0x10
		out	UCSRB,XL
		ldi	XL,0xe6
		out	UCSRC,XL
.endif

.if ctype == mega88
		sts	UCSR0A,const_0
		ldi	XL,0x10
		sts	UCSR0B,XL
		ldi	XL,0x66
		sts	UCSR0C,XL
.endif

;-------------------------------------------------------------------------------
;clear video-RAM and set pointers
;-------------------------------------------------------------------------------
		ldi	XL,0x0f			;color = white
		rcall	libmio_setcol		;set color
		ldi	YL,LOW(libmio_ram)
		ldi	YH,HIGH(libmio_ram)
		ldi	XL,LOW(libmio_vram)	;2
		st	Y+,XL			;2 start vram
		ldi	XL,HIGH(libmio_vram)	;2
		st	Y+,XL			;2 start vram
		ldi	XL,0x16
libmio_init_c:	st	Y+,const_0
		dec	XL
		brne	libmio_init_c
		ldi	YL,LOW(libmio_ram)
		ldi	YH,HIGH(libmio_ram)
		ldi	XL,0xca
		std	Y+libmio_rand1,XL
		ldi	XL,0x57
		std	Y+libmio_rand3,XL
		std	Y+libmio_kflags,const_0
		std	Y+libmio_lastkey,const_0
		std	Y+libmio_envval,const_0
		std	Y+libmio_fastmode,const_0
		ldi	XL,0xff
		std	Y+libmio_serout,XL
		std	Y+libmio_tcnt,const_0
		std	Y+libmio_rcnt,const_0
		std	Y+libmio_tbcnt,const_0
		std	Y+libmio_rbcnt,const_0
		ret

;------------------------------------------------------------------------------
; set color to XL
;------------------------------------------------------------------------------
libmio_setcol:	sts	libmio_ram+libmio_color,XL
		ret

;------------------------------------------------------------------------------
; set color to normal
;------------------------------------------------------------------------------
libmio_setnorm:	sts	libmio_ram+libmio_color,const_1
		ret

;------------------------------------------------------------------------------
; set color to normal
;------------------------------------------------------------------------------
libmio_setinv:	sts	libmio_ram+libmio_color,const_0
		ret

;-------------------------------------------------------------------------------
; Bin8 -> ASCII (dez)
; XL 		= number
;-------------------------------------------------------------------------------
.ifdef use_libmio_outd
libmio_outd:	push	tempreg1		;save registers
		push	XL
		clt

libmio_outd_07:	ldi	tempreg1,0x2f		;"0"-1 	
libmio_outd_08:	inc	tempreg1		;+1
		subi	XL,100			;-100
		brcc	libmio_outd_08		;no overflow
		subi	XL,-100			;+100
		rcall	libmio_nc		;out digit 10^2

libmio_outd_10:	ldi	tempreg1,0x2f		;"0"-1 	
libmio_outd_11:	inc	tempreg1		;+1
		subi	XL,10			;-10
		brcc	libmio_outd_11		;no overflow
		rcall	libmio_nc		;out digit 10^1
		subi	XL,-10			;+10
		
libmio_outd_12:	mov	tempreg1,XL		;LSB digit
		subi	tempreg1,0xd0
		rcall	libmio_char
		pop	XL
		pop	tempreg1
		ret

;-------------------------------------------------------------------------------
; out numeric char (tempreg1)
;-------------------------------------------------------------------------------
libmio_nc:	cpi	tempreg1,0x30		;is zero
		breq	libmio_nc_2		;yes
		set				;set visible flag
libmio_nc_1:	rjmp	libmio_char		;send char
		
libmio_nc_2:	brts	libmio_nc_1		;flag is set
		ret 
.endif

;-------------------------------------------------------------------------------
; clear video-RAM
;-------------------------------------------------------------------------------
libmio_clr:	push_ptr
		clr	libmio_cur_x		;set X-position
		clr	libmio_cur_y		;set Y-position
		ldi	tempreg1,0x20		;space
		lds	XL,libmio_ram+libmio_color
		sbrs	XL,0
		ldi	tempreg1,0x0f
		ldi	YH,HIGH(libmio_vram)	;home position
		ldi	YL,LOW(libmio_vram)	;home position
		ldi	XL,libmio_rows
libmio_cls_01:	ldi	XH,libmio_cols
libmio_cls_02:	st	Y+,tempreg1		;write to RAM
		dec	XH			;decrement counter
		brne	libmio_cls_02		;loop
		dec	XL			;decrement counter
		brne	libmio_cls_01		;loop
		pop_ptr

;-------------------------------------------------------------------------------
; newline and calculate new position
;-------------------------------------------------------------------------------
libmio_newline:	push_ptr
		ldi	XL,libmio_rows
		clr	libmio_cur_x		;begin of line
		inc	libmio_cur_y		;next line
		cp	XL,libmio_cur_y		;beyond screen
		brne	libmio_newlin1
		ldi	XH,(libmio_rows-1)	;last line
		mov	libmio_cur_y,XH
		rcall	libmio_char_04		;scroll screen
libmio_newlin1:	pop_ptr

;-------------------------------------------------------------------------------
; output character and calculate new position (tempreg1=char)
;-------------------------------------------------------------------------------
libmio_space:	ldi	tempreg1,0x20		;set char to space
libmio_char:	push_ptr

		lds	XL,libmio_ram+libmio_color
		sbrc	XL,3
		rjmp	libmio_bchar
		rcall	libmio_calc		;calculate address

;------------------------------------------------------------------------------
; write char tempreg1 to Y
;------------------------------------------------------------------------------
libmio_wchar:	push	XL			;save XL
		ld	XL,Y			;get char at position which we will overwrite
		andi	XL,0x0f			;mask bits
		cpi	XL,0x00			;/16 remains 0?
		breq	libmio_wchar0		;yes->OK
		sts	libmio_ram+libmio_cflag,XL		;store value
libmio_wchar0:	andi	tempreg1,0x7f		;mask bits
		lds	XL,libmio_ram+libmio_color
		sbrs	XL,0
		ori	tempreg1,0x80
		pop	XL
		st	Y,tempreg1

		inc	XL			;next position
		cpi	XL,libmio_cols		;beyond the end of line
		brne	libmio_char_01
		clr	XL			;begin of line
		inc	XH			;of next line
		cpi	XH,libmio_rows		;beyond screen
		brne	libmio_char_01
		ldi	XH,libmio_rows-1	;last line
		movw	libmio_cur_x,XL
		rjmp	libmio_char_03		;we must scroll
libmio_char_01:	movw	libmio_cur_x,XL		;X
libmio_char_02:	pop_ptr

; scroll screen
libmio_char_04:	push_ptr
libmio_char_03:	ldi	YH,HIGH(libmio_vram)	;home position
		ldi	YL,LOW(libmio_vram)	;home position
		ldi	XL,libmio_rows-1	;lines -1
libmio_char_05:	ldi	XH,libmio_cols
libmio_char_06:	ldd	ZL,Y+libmio_cols
		st	Y+,ZL
		dec	XH
		brne	libmio_char_06
		dec	XL
		brne	libmio_char_05
		ldi	XL,libmio_cols
		ldi	ZL,0x20			;space
libmio_char_07:	st	Y+,ZL
		dec	XL
		brne	libmio_char_07
		pop_ptr

libmio_bchar:	mov	XL,libmio_cur_x		;X-position
		mov	XH,libmio_cur_y		;Y-position
		cpi	XL,libmio_cols-3
		brcc	libmio_char_02
		cpi	XH,libmio_rows-5
		brcc	libmio_char_02
		subi	XL,0xfd			;X+3
		mov	libmio_cur_x,XL		;store new X
		subi	XL,3
		lsl	XL			;*2
		lsl	XH			;*2
		push	tempreg4
		push	tempreg5		;temp color
		lds	tempreg5,libmio_ram+libmio_color
		push	tempreg2		;temp value

libmio_bchar_0:	mov	ZL,tempreg1		;char
		ldi	ZH,HIGH(libmio_ctable*2)	;start char ROM
		ldi	YH,10			;10 char lines
		
libmio_bchar_1:	lpm	tempreg1,Z		;get pixline
		ldi	YL,6			;6 pixel
libmio_bchar_2:	mov	tempreg4,tempreg5
		sbrs	tempreg1,7
		add	tempreg4,const_1
		sts	libmio_ram+libmio_color,tempreg4
		rcall	libmio_plot
		inc	XL
		lsl	tempreg1
		dec	YL
		brne	libmio_bchar_2
		subi	XL,6
		inc	XH
		ldi	tempreg2,0x80
		add	ZL,tempreg2
		adc	ZH,const_0
		dec	YH
		brne	libmio_bchar_1
		sts	libmio_ram+libmio_color,tempreg5
		pop	tempreg2
		pop	tempreg5
		pop	tempreg4
		pop_ptr

;-------------------------------------------------------------------------------
; output text from ROM (Z=text,0-terminated)
;-------------------------------------------------------------------------------
.ifdef use_libmio_orom
libmio_orom:	push	tempreg1
libmio_orom_1:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,0x00		;end?
		brne	libmio_orom_2		;no
		pop	tempreg1
		ret				;the end
libmio_orom_2:	rcall	libmio_char		;write char to screen
		rjmp	libmio_orom_1		;next char
.endif

;-------------------------------------------------------------------------------
; output text from ROM (following y,x,0-terminated text)
;-------------------------------------------------------------------------------
.ifdef use_libmio_othis
libmio_othisi:	sts	libmio_ram+libmio_color,const_0	;set invers
libmio_othis:	pop	ZH			;get return address
		pop	ZL
		lsl	ZL			;*2 so we use bytes
		rol	ZH
		push	XL
		push	XH
		lpm	XH,Z+			;get first value (255/Y)
		cpi	XH,0xff			;no position
		breq	libmio_othis_0
		lpm	XL,Z+			;get second value
		rcall	libmio_gxy		;set position
libmio_othis_0:	pop	XH
		pop	XL
		push	tempreg1		;save reg
libmio_othis_1:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,0x00		;end?
		brne	libmio_othis_2		;no
		pop	tempreg1		;restore reg
		sbiw	ZL,1			;point to zero
		lsr	ZH			;/2
		ror	ZL
		adiw	ZL,1			;point to next instruction
		ijmp
libmio_othis_2:	rcall	libmio_char		;write char to screen
		rjmp	libmio_othis_1		;next char
.endif

;-------------------------------------------------------------------------------
; output text from RAM (Z=text,nullterminated)
;-------------------------------------------------------------------------------
.ifdef use_libmio_oram
libmio_oram:	push	tempreg1
libmio_oram_1:	ld	tempreg1,Z+		;get char
		cpi	tempreg1,0		;end?
		brne	libmio_oram_2		;no
		pop	tempreg1
		ret				;the end
libmio_oram_2:	call	libmio_char		;write char to screen
		rjmp	libmio_oram_1		;next char
.endif

;-------------------------------------------------------------------------------
; output text (multiple char) ZL=number
;-------------------------------------------------------------------------------
.ifdef use_libmio_mtext
libmio_mtext:	call	libmio_char		;write char to screen
		dec	ZL
		brne	libmio_mtext
		ret
.endif

;-------------------------------------------------------------------------------
; set cursor to position XL:XH
;-------------------------------------------------------------------------------
.ifdef use_libmio_gotoxy
libmio_gxy:	cpi	XL,libmio_cols
		brcs	libmio_gxy_01
		ldi	XL,libmio_cols-1
libmio_gxy_01:	cpi	XH,libmio_rows
		brcs	libmio_gxy_02
		ldi	XH,libmio_rows-1
libmio_gxy_02:	mov	libmio_cur_x,XL		;save X-position
		mov	libmio_cur_y,XH		;save Y-position
		ret
.endif

;-------------------------------------------------------------------------------
; wait for the end of visible frame
;-------------------------------------------------------------------------------
.ifdef use_libmio_sync
libmio_sync:	push	XL
libmio_sync_1:	mov	XL,vline_l
		cpi	XL,libmio_pixel_rows+1
		breq	libmio_sync_1
libmio_sync_2:	mov	XL,vline_l
		cpi	XL,libmio_pixel_rows+1
		brne	libmio_sync_2
		pop	XL
		ret
.endif

;-------------------------------------------------------------------------------
; wait approx 1ms
;-------------------------------------------------------------------------------
.ifdef use_libmio_wms
libmio_wms:	push	XL
libmio_wms_1:	mov	XL,vline_l
		subi	XL,-16
libmio_wms_2:	cp	XL,vline_l
		brne	libmio_wms_2
		pop	XL
		ret
.endif

;------------------------------------------------------------------------------
;wait for a key
;------------------------------------------------------------------------------
.ifdef use_libmio_waitkey
libmio_waitkey:	ldi	tempreg1,0x00
		sts	libmio_ram+libmio_lastkey,tempreg1
libmio_wkey1:	lds	tempreg1,libmio_ram+libmio_lastkey
		cpi	tempreg1,0x00
		breq	libmio_wkey1
		rcall	libmio_scrs		;check for screenshot
		ret
.endif

;------------------------------------------------------------------------------
;wait for no key
;------------------------------------------------------------------------------
.ifdef use_libmio_waitnokey
libmio_wnokey:	lds	tempreg1,libmio_ram+libmio_keycode
		cpi	tempreg1,0x00
		brne	libmio_wnokey
		ret
.endif

;------------------------------------------------------------------------------
;blink cursor and wait for key (return x,y in X ; adr in Y ; key in tempreg1)
;------------------------------------------------------------------------------
.ifdef use_libmio_cursor
libmio_cursor:	rcall	libmio_calc		;calculate address
libmio_cur1:	ld	XL,Y			;get original char
		mov	XH,XL
		subi	XH,0x80
		ldi	tempreg1,0x00
		sts	libmio_ram+libmio_lastkey,tempreg1	;clear key
libmio_cur_1:	rcall	libmio_scrs
		lds	tempreg1,libmio_ram+libmio_lastkey
		cpi	tempreg1,0x00		;no key?
		brne	libmio_cur_e		;key is found

		lds	tempreg1,libmio_ram+libmio_frames
		cpi	tempreg1,27		;ca. half of fps
		brcc	libmio_cur_2
		st	Y,XL
		rjmp	libmio_cur_1
libmio_cur_2:	st	Y,XH
		rjmp	libmio_cur_1
		
libmio_cur_e:	st	Y,XL			;put back original char
		rjmp	libmio_calc
.endif

;------------------------------------------------------------------------------
;calculate char address to Y
;------------------------------------------------------------------------------
libmio_calc:	movw	XL,libmio_cur_x		;X
libmio_calc1:	ldi	YH,libmio_cols		;chars per line
		mul	XH,YH			;store
		ldi	YH,HIGH(libmio_vram)	;home position
		ldi	YL,LOW(libmio_vram)	;home position
		add	YL,XL			;position+x
		adc	YH,const_0		
		add	YL,r0			;position+(cols*y)
		adc	YH,r1
		ret

;------------------------------------------------------------------------------
;input tempreg1 chars at current position,
; -> Y=address, tempreg1=chars (0xff = abort) 
;------------------------------------------------------------------------------
.ifdef use_libmio_input
libmio_inp:	push_ptr
		movw	libmio_cur_x,XL
		mov	ZH,tempreg1		;max number
		dec	ZH			;-1
		clr	ZL			;current number
		rcall	libmio_calc		;calc pos

libmio_inp_01:	rcall	libmio_cursor		;view cursor
		cpi	tempreg1,0xed		;ESC
		breq	libmio_inp_esc
		cpi	tempreg1,0xea		;ENTER
		breq	libmio_inp_end
		cpi	tempreg1,0xe2		;cursor left
		brne	libmio_inp_02		;branch if not
		cpi	ZL,0x00			;first
		breq	libmio_inp_02		;branch if yes
		dec	libmio_cur_x		;set x pos -1
		dec	ZL			;dec counter
		rjmp	libmio_inp_01		;goto loop

libmio_inp_02:	cpi	tempreg1,0xe3		;cursor right
		brne	libmio_inp_03		;branch if not
		cp	ZL,ZH			;last
		breq	libmio_inp_03		;branch if yes
libmio_inp_04:	inc	libmio_cur_x		;set x pos +1
		inc	ZL			;inc counter
		rjmp	libmio_inp_01		;goto loop

libmio_inp_03:	cpi	tempreg1,0x80		;a char?
		brcc	libmio_inp_01		;no
		st	Y+,tempreg1		
		cp	ZL,ZH			;last
		breq	libmio_inp_01		;branch if yes
		rjmp	libmio_inp_04

libmio_inp_esc:	ldi	ZL,0xff		;break with escape
libmio_inp_end:	mov	tempreg1,ZL		;
		pop_ptr
.endif

;------------------------------------------------------------------------------
;get char on cursor address
;------------------------------------------------------------------------------
.ifdef use_libmio_gchar
libmio_gchar:	rcall	libmio_calc
		ld	tempreg1,Y
		ret
.endif

;------------------------------------------------------------------------------
;get tempreg2 from serial input
;------------------------------------------------------------------------------
libmio_gser:	push	XH
libmio_gser_1:	lds	XH,libmio_ram+libmio_rbcnt	;counter
		cpi	XH,0x00				;ready?
		brne	libmio_gser_1			;loop
		ldi	XH,0x0a				;prepare start
		sts	libmio_ram+libmio_rbcnt,XH	;store start
libmio_gser_2:	lds	XH,libmio_ram+libmio_rbcnt	;counter
		cpi	XH,0x00				;ready?
		brne	libmio_gser_2			;loop
		lds	tempreg2,libmio_ram+libmio_rdat	;get char
libmio_pconv1:	pop	XH
		ret					;thats all

;------------------------------------------------------------------------------
;put tempreg2 to serial output
;------------------------------------------------------------------------------
libmio_pser:	push	XH
libmio_pser_1:	lds	XH,libmio_ram+libmio_tbcnt	;counter
		cpi	XH,0x00				;ready?
		brne	libmio_pser_1			;loop
		sts	libmio_ram+libmio_tdat,tempreg2	;store data
		ldi	XH,0x0b				;prepare 0xff
		sts	libmio_ram+libmio_tbcnt,XH	;store start
		pop	XH
		ret					;thats all

;-------------------------------------------------------------------------------
;scroll inner screen (tempreg1=0 to 3)
;-------------------------------------------------------------------------------
libmio_scrl:	push_ptr
		andi	tempreg1,0x03
		brne	libmio_scrl_10
;scroll up
		ldi	XL,LOW(libmio_vram+2*libmio_cols+2)	;destination
		ldi	XH,HIGH(libmio_vram+2*libmio_cols+2)
		ldi	YL,LOW(libmio_vram+3*libmio_cols+2)	;source
		ldi	YH,HIGH(libmio_vram+3*libmio_cols+2)
		ldi	ZL,libmio_rows-4
libmio_scrl_01:	ldi	ZH,libmio_cols-4
libmio_scrl_02:	ld	tempreg1,Y+
		st	X+,tempreg1
		dec	ZH
		brne	libmio_scrl_02
		adiw	XL,4
		adiw	YL,4
		dec	ZL
		brne	libmio_scrl_01
		ldi	ZH,libmio_cols-4
libmio_scrl_03:	st	X+,const_0
		dec	ZH
		brne	libmio_scrl_03
		rjmp	libmio_scrl_e

;scroll right
libmio_scrl_10:	cpi	tempreg1,0x01		;code for right
		brne	libmio_scrl_20		;no
		ldi	XL,LOW(libmio_vram+libmio_cols*(libmio_rows-2)-2)
		ldi	XH,HIGH(libmio_vram+libmio_cols*(libmio_rows-2)-2)
		ldi	YL,LOW(libmio_vram+libmio_cols*(libmio_rows-2)-3)
		ldi	YH,HIGH(libmio_vram+libmio_cols*(libmio_rows-2)-3)
		ldi	ZL,libmio_rows-4
libmio_scrl_11:	ldi	ZH,libmio_cols-5
libmio_scrl_12:	ld	tempreg1,-Y
		st	-X,tempreg1
		dec	ZH
		brne	libmio_scrl_12
		st	-X,const_0
		sbiw	XL,4
		sbiw	YL,5
		dec	ZL
		brne	libmio_scrl_11
		rjmp	libmio_scrl_e

;scroll down
libmio_scrl_20:	cpi	tempreg1,0x02		;code for down
		brne	libmio_scrl_30		;no
		ldi	XL,LOW(libmio_vram+libmio_cols*(libmio_rows-2)-2)
		ldi	XH,HIGH(libmio_vram+libmio_cols*(libmio_rows-2)-2)
		ldi	YL,LOW(libmio_vram+libmio_cols*(libmio_rows-3)-2)
		ldi	YH,HIGH(libmio_vram+libmio_cols*(libmio_rows-3)-2)
		ldi	ZL,libmio_rows-5
libmio_scrl_21:	ldi	ZH,libmio_cols-4
libmio_scrl_22:	ld	tempreg1,-Y
		st	-X,tempreg1
		dec	ZH
		brne	libmio_scrl_22
		sbiw	XL,4
		sbiw	YL,4
		dec	ZL
		brne	libmio_scrl_21
		ldi	ZH,libmio_cols-4
libmio_scrl_23:	st	-X,const_0
		dec	ZH
		brne	libmio_scrl_23
		rjmp	libmio_scrl_e
;scroll left
libmio_scrl_30:	ldi	XL,LOW(libmio_vram+libmio_cols*2+2)
		ldi	XH,HIGH(libmio_vram+libmio_cols*2+2)
		ldi	YL,LOW(libmio_vram+libmio_cols*2+3)
		ldi	YH,HIGH(libmio_vram+libmio_cols*2+3)
		ldi	ZL,libmio_rows-4
libmio_scrl_31:	ldi	ZH,libmio_cols-5
libmio_scrl_32:	ld	tempreg1,Y+
		st	X+,tempreg1
		dec	ZH
		brne	libmio_scrl_32
		st	X+,const_0
		adiw	YL,5
		adiw	XL,4
		dec	ZL
		brne	libmio_scrl_31
libmio_scrl_e:	pop_ptr

;-------------------------------------------------------------------------------
; screenshot
;-------------------------------------------------------------------------------
libmio_scrs:	push_ptr
		lds	ZL,libmio_ram+libmio_kflags
		sbrs	ZL,2
		rjmp	libmio_scrs2
		sts	libmio_ram+libmio_kflags,const_0
		sts	libmio_ram+libmio_lastkey,const_0
		ldi	XL,LOW(libmio_vram)
		ldi	XH,HIGH(libmio_vram)
		ldi	ZH,libmio_rows
libmio_scrs1:	ldi	ZL,libmio_cols
libmio_scrs_1:	call	mem_cout		;out char
		dec	ZL
		brne	libmio_scrs_1
		rcall	libmio_snl
		dec	ZH
		brne	libmio_scrs1
libmio_scrs2:	pop_ptr

libmio_snl:	push	ZL
		push	tempreg2
		lds	ZL,libmio_ram+libmio_sysconf
		ldi	tempreg2,0x0d		;CR
		sbrc	ZL,0
		rcall	libmio_pser
		ldi	tempreg2,0x0a		;LF
		sbrc	ZL,1
		rcall	libmio_pser
		pop	tempreg2
		pop	ZL
		ret

;-------------------------------------------------------------------------------
; pseudorandom generator
;-------------------------------------------------------------------------------
;.ifdef use_libmio_psrg
libmio_psrg:	push_ptr
		rcall	libmio_rand
		pop_ptr
;.endif

libmio_rand:	ldd	ZL,Y+libmio_rand1
		ldd	XL,Y+libmio_rand2
		ldd	XH,Y+libmio_rand3
		ldd	ZH,Y+libmio_rand4
		clr	YL
		sbrc	ZL,2
		com	YL
		sbrc	ZL,5
		com	YL
		sbrc	XL,4
		com	YL
		sbrc	ZH,7
		com	YL
		rol	YL
		rol	ZL
		rol	XL
		rol	XH
		rol	ZH
		ldi	YL,LOW(libmio_ram)	;1 set ptr
		std	Y+libmio_rand1,ZL
		std	Y+libmio_rand2,XL
		std	Y+libmio_rand3,XH
		std	Y+libmio_rand4,ZH
		ret

.include	"libmio/graph.asm"
