#!/usr/bin/perl
#################################################################################
#										#
# screenshot tool for avr-chipbasic8						#
# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
#										#
# This program is free software; you can redistribute it and/or			#
# modify it under the terms of the GNU General Public License			#
# as published by the Free Software Foundation; either version 3		#
# of the License, or (at your option) any later version.			#
#										#
# This program is distributed in the hope that it will be useful,		#
# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
# General Public License for more details.					#
#										#
# You should have received a copy of the GNU General Public			#
# License along with this library; if not, write to the				#
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
# Boston, MA 02111-1307, USA.							#
#										#
#################################################################################
$serdevice="/dev/ttyS0";

###############################################################################
# get pixel value from tab
###############################################################################
sub get_pixel
{
    $nb=0;
    if ($z =~ /^\./) {$nb=0}
    elsif ($z =~ /^x/) {$nb=1}
    $z=$';
    return $nb;
}

###############################################################################
# get nibble data from matrix
###############################################################################
sub get_nibble
{
    $nb=0;
    if ($z =~ /^A/) {$nb=0}
    elsif ($z =~ /^B/) {$nb=1}
    elsif ($z =~ /^C/) {$nb=2}
    elsif ($z =~ /^D/) {$nb=3}
    elsif ($z =~ /^E/) {$nb=4}
    elsif ($z =~ /^F/) {$nb=5}
    elsif ($z =~ /^G/) {$nb=6}
    elsif ($z =~ /^H/) {$nb=7}
    elsif ($z =~ /^I/) {$nb=8}
    elsif ($z =~ /^J/) {$nb=9}
    elsif ($z =~ /^K/) {$nb=10}
    elsif ($z =~ /^L/) {$nb=11}
    elsif ($z =~ /^M/) {$nb=12}
    elsif ($z =~ /^N/) {$nb=13}
    elsif ($z =~ /^O/) {$nb=14}
    elsif ($z =~ /^P/) {$nb=15}
    $z=$';
    return $nb;
}

###############################################################################
# read data from chip
###############################################################################
sub get_data
{
    open (SER, $serdevice);
    for($lineno=0;$lineno<23;$lineno++)
    {
	$line[$lineno]=<SER>;
	print "*";
    }
    close(SER);
}

###############################################################################
# convert strings to char array
###############################################################################
sub fillarray
{
    for ($y=0;$y<23;$y++)
    {
	$z=$line[$y];
	for ($x=0;$x<30;$x++)
	{
	    $hi=&get_nibble();
	    $lo=&get_nibble();
	    $vcell[30*$y+$x]=16*$hi+$lo;
	}
    }
}

###############################################################################
# write xpm image (normal mode)
###############################################################################
sub writexpm
{
    open (WTAB, ">temp.xpm");
    print WTAB "static char *screenshot_xpm[] ={\n";
    print WTAB "\"370 240 2 1\",\n";
    print WTAB "\". c #000000\",\n";
    print WTAB "\"W c #ffffff\",\n";

#Black border (upper)
    for($pline=0;$pline<5;$pline++)
    {
	print WTAB "\""; 
	for($chr=0;$chr<370;$chr++)
	{
    	    print WTAB "."; 
	}
	print WTAB "\",\n";
    }

    for($cline=0;$cline<23;$cline++)
    {
	for($pline=0;$pline<10;$pline++)
	{
    	    print WTAB "\"....."; 
	    for($chr=0;$chr<30;$chr++)
	    {
		$vchar=$vcell[30*$cline+$chr];
		$vg="W";
		$hg=".";
		if($vchar>127)
		{
		    $vg=".";
		    $hg="W";
		    $vchar-=128;
		}

		for($pix=0;$pix<6;$pix++)
		{
		    $pchar=$hg;
		    if ($charcell[6*$pline+60*$vchar+$pix]>0)
		    {
			$pchar=$vg;    
	    	    }
		    print WTAB $pchar.$pchar;
		}
	    }    
    	    print WTAB ".....\",\n";
	}
    }

#Black border (lower)
    for($pline=0;$pline<5;$pline++)
    {
	print WTAB "\""; 
	for($chr=0;$chr<370;$chr++)
	{
    	    print WTAB "."; 
	}
	    if ($pline<5)
	    {
    	        print WTAB "\",\n";
	    }
	    else
	    {
        	print WTAB "\"\n";
	    }    
    }
    
    print WTAB "};\n"; 
    close(WTAB);
}

###############################################################################
# get filename and set color array
###############################################################################
$fname=$ARGV[0];

$ct[0]=".";
$ct[1]="W";

###############################################################################
# read character cells from fnt0
###############################################################################
$cline=0;
$count=-1;
open (RTAB, "../generator/ctab.dat");
while(<RTAB>)
{
    $z=$_;
    chomp($z);
    if ($z =~ /cell/) 
    {
	$cline=0;
	$count++;
    }
    elsif (length($z)>4)
    {
	$charcell[6*($cline)+60*$count+0]=&get_pixel();
	$charcell[6*($cline)+60*$count+1]=&get_pixel();
	$charcell[6*($cline)+60*$count+2]=&get_pixel();
	$charcell[6*($cline)+60*$count+3]=&get_pixel();
	$charcell[6*($cline)+60*$count+4]=&get_pixel();
	$charcell[6*($cline)+60*$count+5]=&get_pixel();
	$cline++;
    }
}
close(RTAB);

###############################################################################
# set serial port and read data from chip
###############################################################################
system `stty 1200 cs8 -parenb -echo -crtscts -ixon < /dev/ttyS0`;
print "AVR-ChipBasic8 Screenshot loader\n";
print "reading data...";
&get_data();
print "done\n";

###############################################################################
# do it
###############################################################################
&fillarray();
&writexpm();

###############################################################################
# convert xpm image to whatever you want
###############################################################################
system`convert temp.xpm $fname`;
system`rm temp.xpm`;
