;################################################################################
;#										#
;# chipbasic8									#
;# single chip basic computer with atmega 8/16/88				#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public				#
;# License as published by the Free Software Foundation; either			#
;# version 3 of the License, or (at your option) any later version.		#
;#										#
;# This library is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################
.set		kbd_de = 1
.set		kbd_en = 2
.set		pal = 1
.set		ntsc = 2
.set		mega8 = 8
.set		mega16 = 16
.set		mega88 = 88

;------------------------------------------------------------------------------
;configuration
;------------------------------------------------------------------------------
.include	"config"

.CSEG
;------------------------------------------------------------------------------
;IO and memory definitions
;------------------------------------------------------------------------------
.if ctype == 8
.include	"includes/m8def.inc"
.equ		vport = PORTD			;video port
.equ		ioport_in = PINC
.equ		ioport_out = PORTC
.equ		ioport_ddr = DDRC
.equ		seriport = PIND
.equ		seripin = 5
.equ		seroport = PORTB
.equ		seropin = 4
.equ		sercport = PORTB
.equ		sercpin = 5
.equ		modeport = PIND
.equ		modepin = 2
.equ		libmio_ram	= 0x060		;32 bytes libmio system RAM
.equ		libmio_vram	= 0x088		;690 bytes video RAM
.equ		expar_ros	= 0x35e		;16 bytes operand stack
.equ		varspace	= 0x35e		;26 bytes basic variables
.equ		bas_buf		= 0x378		;6 bytes basic buffer to 380
.equ		bas_ram		= 0x400		;8 bytes basic system RAM
.equ		bas_stacks	= 0x408		;40 bytes basic stacks
.equ		stack		= 0x45f		;48 bytes system stack
.endif

.if ctype == mega16
.include	"includes/m16def.inc"
.equ		vport = PORTC			;video port
.equ		vddr = DDRC
.equ		ioport_in = PINA
.equ		ioport_out = PORTA
.equ		ioport_ddr = DDRA
.equ		seriport = PIND
.equ		seripin = 3
.equ		seroport = PORTD
.equ		seropin = 3
.equ		sercport = PORTB
.equ		sercpin = 3
.equ		modeport = PINC
.equ		modepin = 4
.equ		libmio_ram	= 0x060		;32 bytes libmio system RAM
.equ		libmio_vram	= 0x088		;720 bytes video RAM
.equ		expar_ros	= 0x35e		;16 bytes operand stack
.equ		varspace	= 0x35e		;26 bytes basic variables
.equ		bas_buf		= 0x378		;6 bytes basic buffer to 380
.equ		bas_ram		= 0x400		;8 bytes basic system RAM
.equ		bas_stacks	= 0x408		;40 bytes basic stacks
.equ		stack		= 0x45f		;48 bytes system stack
.endif

.if ctype == mega88
.include	"includes/m88def.inc"
.equ		vport = PORTD			;video port
.equ		vddr = DDRD
.equ		csport = PORTB			;composite sync port
.equ		csddr = DDRB
.equ		csbit = 1
.equ		ioport_in = PINC
.equ		ioport_out = PORTC
.equ		ioport_ddr = DDRC
.equ		seriport = PIND
.equ		seripin = 5
.equ		seroport = PORTB
.equ		seropin = 4
.equ		sercport = PORTB
.equ		sercpin = 5
.equ		modeport = PIND
.equ		modepin = 2
.equ		libmio_ram	= 0x100		;36 bytes libmio system RAM
.equ		libmio_vram	= 0x124		;690 bytes video RAM
.equ		expar_ros	= 0x404		;16 bytes operand stack
.equ		varspace	= 0x404		;26 bytes basic variables
.equ		bas_buf		= 0x41e		;6 bytes basic buffer to 380
.equ		bas_ram		= 0x424		;8 bytes basic system RAM
.equ		bas_stacks	= 0x42c		;40 bytes basic stacks
.equ		stack		= 0x4ff		;48 bytes system stack
.endif

;------------------------------------------------------------------------------
;register definitions
;------------------------------------------------------------------------------
.def		const_0 = r2
.def		const_1 = r3
.def		vline_l = r8
.def		vline_h = r9
.def		bas_prog = r12
.def		bas_line = r13
.def		bas_statement = r14
.def		color = r15

.def		tempreg1 = r20
.def		tempreg2 = r21
.def		tempreg3 = r22
.def		tempreg4 = r23
.def		tempreg5 = r4
.def		tempreg6 = r5
.def		tempreg7 = r6
.def		tempreg8 = r7

.def		ereg = r24
.def		ctrl = r25

.set		bas_lines=20
.set		mem_linelen=25
.set		bas_stackdepth=10


.include	"libmio/definitions.asm"
.include	"macros.inc"


.org 0x0000
		rjmp	start
.if ctype == mega8
    .org OC1Baddr
.endif
.if ctype == mega16
    .org OC1Baddr
.endif
.if ctype == mega88
    .org OC1Aaddr
.endif

		libmio_vint

start:		cli
		ldi	XL,0x00
		mov	const_0,XL
		ldi	XL,0x01
		mov	const_1,XL
		ldi	XL,HIGH(stack)
		out	SPH,XL
		ldi	XL,LOW(stack)
		out	SPL,XL
		libmio_init
		libi2c_init
		ldi	YL,0xff
		ldi	YH,0x01
		libeep_read
		sts	libmio_ram+libmio_sysconf,tempreg1
		clr	ereg
		sei
		rcall	intro_main
		lds	XL,libmio_ram+libmio_lastkey
		cpi	XL,0
		brne	start_01
		jmp	autorun_main
start_01:	mov	bas_prog,const_0
		mov	bas_line,const_0
		mov	bas_statement,const_0
		rjmp	editor_main

push_ptr:	pop	r0
		pop	r1
		push	ZH
		push	ZL
		push	YH
		push	YL
		push	XH
		push	XL
		push	r1
		push	r0
		ret

.macro push_ptr
call		push_ptr
.endm

pop_ptr:	pop	XL
		pop	XH
		pop	YL
		pop	YH
		pop	ZL
		pop	ZH
		ret

.macro pop_ptr
jmp		pop_ptr
.endm

.include	"modules/intro.asm"
.include	"modules/file.asm"
.include	"modules/editor.asm"
.include	"modules/exparser.asm"
.include	"modules/interpreter.asm"
.include	"modules/autorun.asm"

.include	"libs.inc"
.org 	0xb40					;program 1
ddstab:
.include	"libmio/snd.inc"

.org 	0xb80					;program 1
bas_programs:
.include	"demoprog/prog1.inc"
		
.org	0xc80
libmio_ctable:
.include	"libmio/ctable.inc"

;------------------------------------------------------------------------------
;rww section
;------------------------------------------------------------------------------
.if ctype == mega8
    .org	0x0f00
    .include	"modules/memory.asm"
    .org 	0x0f74					;keyboard table
.endif
.if ctype == mega16
    .org	0x1f00
    .include	"modules/memory.asm"
    .org 	0x1f74					;keyboard table
.endif
.if ctype == mega88
    .org	0x0f00
    .include	"modules/memory.asm"
    .org 	0x0f74					;keyboard table
.endif

libmio_ktable:

.if kbd_type == kbd_de
    .include	"libmio/keytable_de.inc"
.endif
.if kbd_type == kbd_en
    .include	"libmio/keytable_us.inc"
.endif
