;################################################################################
;#										#
;# avr-chipbasic8 - single chip basic computer with ATmega8			#
;# expression parser								#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;##############################################################################
; the basic parser
; Z= pointer in flash
;##############################################################################
expar0:		movw	ZL,tempreg7		;restore pointer
expar:		push	XH			;save register
		push	YL
		push	YH
		ldi	YL,LOW(expar_ros)	;value stack
		ldi	YH,HIGH(expar_ros)	;
		st	-Y,const_0		;write zero value
		push	const_0			;write operand stack stopper
		clr	XL			;OP
		clr	tempreg4		;clear status (last=op)

;------------------------------------------------------------------------------
; get character from string
;------------------------------------------------------------------------------
expar_entry:	rcall	bas_setupper0		;get char 1
		cp	ZL,ctrl			;end of line
		breq	expar_end_1		;yes

		cpi	tempreg1,0x20		;space
		breq	expar_entry
		sbiw	ZL,1			;correct pointer

expar_loop:	rcall	bas_setupper0		;get next char
		cp	ZL,ctrl			;EOL
		breq	expar_end_1		;yes

;------------------------------------------------------------------------------
; check for the end
;------------------------------------------------------------------------------
expar_end:	cpi	tempreg1,0x21		;space
		brcs	expar_end_1		;yes
		cpi	tempreg1,0x27		;comment
		breq	expar_end_1		;yes
		cpi	tempreg1,','		;comma
		breq	expar_end_1		;yes
		cpi	tempreg1,':'		;colon
		breq	expar_end_1		;yes
		cpi	tempreg1,0x3b		;semicolon
		breq	expar_end_1		;yes
		rjmp	expar_end_e		;go on

expar_end_1:	pop	tempreg2		;get last operand
		cpi	tempreg2,0x00		;end?
		breq	expar_end_2		;->end
		cpi	tempreg2,0x1f		;a open function?
		brcs	expar_errend
		rcall	expar_doar
		rjmp	expar_end_1

expar_end_2:	ld	XL,Y+			;get result
		sbiw	ZL,1			;corect Z-pointer to char after expr
		pop	YH
		pop	YL
		pop	XH
		ret

expar_errend:	ldi	ereg,err_expr
expar_errend_1:	pop	tempreg2		;get last operand
		cpi	tempreg2,0x00		;end?
		brne	expar_errend_1		;no
		pop	YH
		pop	YL
		pop	XH
		pop	tempreg2		;kill stack
		pop	tempreg2		;kill stack
		ret
expar_end_e:

;------------------------------------------------------------------------------
; check for a hexadecimal number
;------------------------------------------------------------------------------
expar_hex:	cpi	tempreg1,'$'		;hex numbers start with $
		brne	expar_hex_e		;no
		clr	XL			;clear value

expar_hex_1:	rcall	bas_ispace		;get next and set to uppercase
		set				;accept hex
		rcall	expar_gnum		;get number
		cpi	tempreg1,0x10
		brcc	expar_hex_4
expar_hex_3:	swap	XL			;*16
		andi	XL,0xf0
		add	XL,tempreg1
		rjmp	expar_hex_1
expar_hex_4:	sbiw	ZL,1			;set ptr to failed char
		rjmp	expar_wval
expar_hex_e:

;------------------------------------------------------------------------------
; check for a decimal number
;------------------------------------------------------------------------------
expar_dec:	clt				;accept only decimal
		rcall	expar_gnum		;get number
		cpi	tempreg1,0x0a		;
		brcc	expar_dec_e		;next check
		clr	XL			;clear value
;mult result with 10 and add actual value
expar_dec_1:	ldi	XH,10			;factor
		mul	XL,XH			;value * 10
		mov	XL,r0
		add	XL,tempreg1
		lpm	tempreg1,Z+		;get next char
		rcall	expar_gnum
		cpi	tempreg1,0x0a		;"9"+1
		brcc	expar_dec_2		;not a digit
		rjmp	expar_dec_1		;add this digit
expar_dec_2:	sbiw	ZL,1			;set ptr to failed char
		rjmp	expar_wval
expar_dec_e:

;------------------------------------------------------------------------------
; a variable "A...Z"
;------------------------------------------------------------------------------
expar_var:	rcall	bas_cchar		;A-Z?
		brts	expar_fun_e		;no
		mov	tempreg2,tempreg1

		lpm	tempreg1,Z+		;get next char
		rcall	bas_cchar		;A-Z?
		brtc	expar_var_e
		sbiw	ZL,1
		ldi	XL,LOW(varspace-65)	;pointer to vars
		ldi	XH,HIGH(varspace-65)
		add	XL,tempreg2		;set actual pointer
		adc	XH,const_0
		ld	XL,X+			;get value
		rjmp	expar_wval		;OK
expar_var_e:

;------------------------------------------------------------------------------
; multichar (2) ops
;------------------------------------------------------------------------------
expar_fun:	mov	r16,tempreg2		;copy first char
		mov	r17,tempreg1		;copy second char
		lpm	tempreg1,Z+
		cpi	tempreg1,'('		;paranthesis?
		breq	expar_fun_2

expar_fun_1:	rjmp	expar_errend		;end with error

expar_fun_2:	movw	tempreg7,ZL			;save pointer to tempreg7/8
		ldi	ZH,HIGH(expar_keywords*2)	;keyword table
		ldi	ZL,LOW(expar_keywords*2)	;
		ldi	XL,0x02			;first token-1
expar_fun_3:	lpm	tempreg1,Z+		;get chars
		lpm	tempreg2,Z+
		inc	XL			;token+1
		cpi	tempreg1,0x00		;end
		breq	expar_fun_1		;unknown->error
		cp	tempreg1,r16		;check char1
		brne	expar_fun_3
		cp	tempreg2,r17		;check char2
		brne	expar_fun_3
		push	XL			;push OP
		mov	tempreg4,XL		;set status
		clr	XL			;clear operand
		movw	ZL,tempreg7		;restore pointer
		rjmp	expar_loop		;next char
expar_fun_e:

;------------------------------------------------------------------------------
; check for parantheses
;------------------------------------------------------------------------------
expar_parao:	cpi	tempreg1,'('		;"("
		brne	expar_parac		;no
		ldi	tempreg4,0x01		;code for left paranthesis
		push	tempreg4		;push to stack
		clr	XL			;clear operand
		rjmp	expar_loop		;next character

;closed paranthese
expar_parac:	cpi	tempreg1,')'		;")"
		brne	expar_parac_e		;no

expar_parac_1:	pop	tempreg2		;get last operator
		cpi	tempreg2,0x01		;left paranthesis
		brne	expar_parac_3		;yes
expar_parac_2:	clr	XL			;clear operand
		ldi	tempreg4,0x02		;code for right paranthesis
		rjmp	expar_loop		;next character

expar_parac_3:	cpi	tempreg2,0x03		;CF(
		brne	expar_parac_6		;not
		rjmp	expar_cflag		;jump to function

expar_parac_6:	cpi	tempreg2,0x04		;RV(
		brne	expar_parac_7		;not
		rjmp	expar_rnd		;jump to function

expar_parac_7:	cpi	tempreg2,0x05		;AD(
		brne	expar_parac_8		;not
		rjmp	expar_adc		;jump to function

expar_parac_8:	cpi	tempreg2,0x06		;IN(
		brne	expar_parac_9		;not
		rjmp	expar_din		;jump to function

expar_parac_9:	cpi	tempreg2,0x07		;EP(
		brne	expar_parac_a		;not
		rjmp	expar_epeek		;jump to function

expar_parac_a:	cpi	tempreg2,0x08		;KY(
		brne	expar_parac_aa		;not
		rjmp	expar_key		;jump to function

expar_parac_aa:	cpi	tempreg2,0x09		;NO(
		brne	expar_parac_b		;not
		rjmp	expar_not		;jump to function

expar_parac_b:	cpi	tempreg2,0x00		;end
		breq	expar_parac_c		;no open->error
		rcall	expar_doar		;do arithmetics
		cpi	ereg,0x00		;error?
		breq	expar_parac_1		;next
expar_parac_c:	rjmp	expar_errend

expar_parac_e:

;------------------------------------------------------------------------------
; check if last was an operand
;------------------------------------------------------------------------------
expar_chkop:	cpi	tempreg4,0x7f		;upper border for operators
		brcc	expar_chkop_e		;last was no operator
		cpi	tempreg4,0x0f		;lower border for operators
		brcs	expar_chkop_e
		rjmp	expar_errend
expar_chkop_e:

;------------------------------------------------------------------------------
; a comparsion
;------------------------------------------------------------------------------
expar_comp:	cpi	tempreg1,'<'		;"<"
		brne	expar_comp_1
		ldi	tempreg1,0x21		;code for less
		rjmp	expar_doop

expar_comp_1:	cpi	tempreg1,'>'		;">"
		brne	expar_comp_2		;no
		ldi	tempreg1,0x22		;code for greater
		rjmp	expar_doop

expar_comp_2:	cpi	tempreg1,'='		;"="
		brne	expar_comp_e		;no
		ldi	tempreg1,0x23		;code for equal
		rjmp	expar_doop
expar_comp_e:

;------------------------------------------------------------------------------
; an arithmetic operator 
;------------------------------------------------------------------------------
expar_arith:	cpi	tempreg1,'+'		;"+"
		brne	expar_arith_1
		ldi	tempreg1,0x30		;code for +
		rjmp	expar_doop

expar_arith_1:	cpi	tempreg1,'-'		;"-"
		brne	expar_arith_2
		ldi	tempreg1,0x31		;code for sub
		rjmp	expar_doop

expar_arith_2:	cpi	tempreg1,'#'		;OR
		brne	expar_arith_3
		ldi	tempreg1,0x32		;code for or
		rjmp	expar_doop

;next level
expar_arith_3:	cpi	tempreg1,'*'		;"*"
		brne	expar_arith_4
		ldi	tempreg1,0x40		;code for mult
		rjmp	expar_doop
			
expar_arith_4:	cpi	tempreg1,'/'		;"/"
		brne	expar_arith_5
		ldi	tempreg1,0x41		;code for div
		rjmp	expar_doop

expar_arith_5:	cpi	tempreg1,'&'		;"&"
		brne	expar_arith_6
		ldi	tempreg1,0x42		;code for and
		rjmp	expar_doop

expar_arith_6:	cpi	tempreg1,'%'		;"%"
		brne	expar_arith_e
		ldi	tempreg1,0x43		;code for modulo
		rjmp	expar_doop

expar_arith_e:

;------------------------------------------------------------------------------
; unknown...
;------------------------------------------------------------------------------
		rjmp	expar_errend		;no useable char->error

;------------------------------------------------------------------------------
; do operator function 
;------------------------------------------------------------------------------
expar_doop:	pop	tempreg2		;get last operator from stack
	    	mov	XL,tempreg1		;recent operator
		mov	XH,tempreg2		;last operator
		andi	XL,0xf0			;we need only priority
		andi	XH,0xf0			;
		cp	XH,XL			;compare last with recent
		brcs	expar_doop_2		;branch if last is not less

expar_doop_1:	rcall	expar_doar		;do last operation
		cpi	ereg,0x00		;error?
		breq	expar_doop		;no
		rjmp	expar_errend

;write last and recent operator on stack
expar_doop_2:	push	tempreg2		;put last on stack
		push	tempreg1		;put recent on stack
		clr	XL			;clear operand
		mov	tempreg4,tempreg1	;set status
		rjmp	expar_loop		;get next char

;------------------------------------------------------------------------------
; execute function with 2 parameters 
;------------------------------------------------------------------------------
expar_doar:	ld	XL,Y+			;get value of TOS
		ld	XH,Y+			;get value of SEC

		cpi	tempreg2,0x21		;<
		brne	expar_doar_01
		rcall	expar_gt
		rjmp	expar_doar_y

expar_doar_01:	cpi	tempreg2,0x22		;>
		brne	expar_doar_02
		rcall	expar_lt
		rjmp	expar_doar_y

expar_doar_02:	cpi	tempreg2,0x23		;=
		brne	expar_doar_03
		rcall	expar_eq
		rjmp	expar_doar_y

expar_doar_03:	cpi	tempreg2,0x30		;+
		brne	expar_doar_04
		add	XL,XH
		rjmp	expar_doar_y
		
expar_doar_04:	cpi	tempreg2,0x31		;-
		brne	expar_doar_05
		sub	XH,XL
		mov	XL,XH
		rjmp	expar_doar_y

expar_doar_05:	cpi	tempreg2,0x42		;&
		brne	expar_doar_06
		and	XL,XH
		rjmp	expar_doar_y

expar_doar_06:	cpi	tempreg2,0x40		;*
		brne	expar_doar_16
		mul	XL,XH
		mov	XL,r0
		rjmp	expar_doar_y		;put TOS

expar_doar_16:	cpi	tempreg2,0x43		;% modulo
		brne	expar_doar_07
		rcall	expar_div
		mov	XL,r18
		rjmp	expar_doar_y		;put TOS

expar_doar_07:	cpi	tempreg2,0x32		;#
		brne	expar_doar_08
		or	XL,XH
		rjmp	expar_doar_y		;put TOS

expar_doar_08:	rcall	expar_div		;call division
		rjmp	expar_doar_y		;put TOS

expar_doar_y:	st	-Y,XL			;write result to stack
		ret

;-------------------------------------------------------------------------------
; 8 Bit RND
;-------------------------------------------------------------------------------
expar_rnd:	libmio_random
		lds	XH,libmio_ram+libmio_rand2		;get a new value
		ld	XL,Y+
		mul	XL,XH
		mov	XL,r1
expar_rnd_1:	st	-Y,XL
		ldi	tempreg4,0x02		;closed paranthese
		clr	XL			;clear operand
		rjmp	expar_loop

;-------------------------------------------------------------------------------
; analog input
;-------------------------------------------------------------------------------
.if ctype == mega8
expar_adc:	ld	XL,Y+			;get value of TOS
		andi	XL,0x03			;4 channels
		ori	XL,0x60

		out	ADMUX,XL
		in	XL,ADCSR
		ori	XL,0xc7			;set ADSC+ADEN
		out	ADCSR,XL
expar_adc1:	in	XL,ADCSR
		sbrc	XL,6
		rjmp	expar_adc1
		in	XL,ADCH
		rjmp	expar_rnd_1
.endif

.if ctype == mega16
expar_adc:	ld	XL,Y+			;get value of TOS
		andi	XL,0x03			;4 channels
		ori	XL,0x60
		out	ADMUX,XL
		in	XL,ADCSR
		ori	XL,0xc7			;set ADSC+ADEN
		out	ADCSR,XL
expar_adc1:	in	XL,ADCSR
		sbrc	XL,6
		rjmp	expar_adc1
		in	XL,ADCH
		rjmp	expar_rnd_1
.endif

.if ctype == mega88
expar_adc:	ld	XL,Y+			;get value of TOS
		andi	XL,0x03			;4 channels
		ori	XL,0x60
		sts	ADMUX,XL
		lds	XL,ADCSRA
		ori	XL,0xc7			;set ADSC+ADEN
		sts	ADCSRA,XL
expar_adc1:	lds	XL,ADCSRA
		sbrc	XL,6
		rjmp	expar_adc1
		lds	XL,ADCH
		rjmp	expar_rnd_1
.endif

;-------------------------------------------------------------------------------
; digital input
;-------------------------------------------------------------------------------
expar_din:	ld	XL,Y+			;get value of TOS
		ld	XL,Y+			;get low value of TOS
		andi	XL,0x03			;only 4 inputs
		inc	XL
		in	XH,ioport_in		;read
expar_din_1:	lsr	XH
		dec	XL
		brne	expar_din_1
		clr	XL
		rol	XL
		andi	XL,0x01
		rjmp	expar_rnd_1

;-------------------------------------------------------------------------------
;epeek
;-------------------------------------------------------------------------------
expar_epeek:	ld	XL,Y+			;get value of TOS
		push	YL
		push	YH
		mov	YL,XL
		clr	YH
		libeep_read			;get byte
		pop	YH
		pop	YL
		mov	XL,tempreg1
		rjmp	expar_rnd_1

;-------------------------------------------------------------------------------
;not
;-------------------------------------------------------------------------------
expar_not:	ld	XL,Y+			;get value of TOS
		com	XL
		rjmp	expar_rnd_1

;-------------------------------------------------------------------------------
;collision Flag
;-------------------------------------------------------------------------------
expar_cflag:	ld	XL,Y+			;get value of TOS (dummy)
		lds	XL,libmio_ram+libmio_cflag	;get collision flag
		rjmp	expar_rnd_1

;-------------------------------------------------------------------------------
; key
;-------------------------------------------------------------------------------
expar_key:	lds	r18,libmio_ram+libmio_kbdstate
		lds	r19,libmio_ram+libmio_keycode
		ld	r16,Y+			;get low value of TOS
		clr	XL			;neutral value
		andi	r16,0x07		;mask bits
		brne	expar_ke1
		mov	XL,r19
		rjmp	expar_rnd_1

expar_ke1:	cpi	r16,0x01		;wait for char
		brne	expar_ke2
		libmio_waitkey
		mov	XL,tempreg1
		rjmp	expar_rnd_1

expar_ke2:	cpi	r16,0x02
		brne	expar_ke3		;jump if no lshift
		sbrc	r18,0			;left shift
		inc	XL
		sbrc	r18,2			;left ctrl
		dec	XL
		rjmp	expar_rnd_1

expar_ke3:	cpi	r16,0x03
		brne	expar_ke4		;jump if no ctrl
		sbrc	r18,1			;right shift
		inc	XL
		sbrc	r18,3			;right ctrl
		dec	XL
		rjmp	expar_rnd_1

expar_ke4:	cpi	r16,0x04
		brne	expar_ke5		;jump if no ctrl
		sbrc	r18,3			;right ctrl
		inc	XL
		sbrc	r18,2			;left ctrl
		dec	XL
		rjmp	expar_rnd_1

expar_ke5:	cpi	r16,0x05
		brne	expar_ke6		;jump if no ctrl
		cpi	r19,0xe2		;left arrow
		brne	expar_ke51
		dec	XL
expar_ke51:	cpi	r19,0xe3		;right arrow
		brne	expar_ke52
		inc	XL
expar_ke52:	rjmp	expar_rnd_1

expar_ke6:	libmio_waitkey
		cpi	tempreg1,0xea		;Enter
		brne	expar_ke6
		rjmp	expar_rnd_1

;-------------------------------------------------------------------------------
; 8 Bit unsigned division
; XH / XL => XL + remainder r18
;-------------------------------------------------------------------------------
expar_div:	cpi	XL,0x00			;check for divisor zero
		brne	expar_div_1		;jump if no zero
		ldi	ereg,3			;set error
		ret				;return (error)

expar_div_1:	clr	r18			;clear remainder L
		clr	r19			;clear remainder H
		clr	r20			;clear carry source
		ldi	r21,9			;loop counter

expar_div_2:	rol	r20
		rol	XH			;lshift dividend
		dec	r21			;loop counter
		breq	expar_div_3		;end of loop
		rol	r18			;shift into remainder
		ldi	r20,0xff		;set carry source
		sub	r18,XL			;sub divisor
		brcc	expar_div_2		;branch if no borrow
		add	r18,XL			;add divisor to restore
		ldi	r20,0x00		;clear carry source
		rjmp	expar_div_2		;goto loop

expar_div_3:	mov	XL,XH			;copy result
expar_div_4:	ret				;thats all

;-------------------------------------------------------------------------------
; 8 Bit equal (return 1 if XL == XH)
;-------------------------------------------------------------------------------
expar_eq:	cp	XL,XH			;compare
		brne	expar_eq1
		ldi	XL,0x01			;result is 1
		ret
expar_eq1:	clr	XL			;result is 0
		ret

;-------------------------------------------------------------------------------
; 8 Bit greater (return 1 if XL > XH)
;-------------------------------------------------------------------------------
expar_gt:	cp	XH,XL			;SEC-TOS
		brcc	expar_eq1
expar_gt1:	ldi	XL,0x01			;result is 1
		ret

;-------------------------------------------------------------------------------
; 8 Bit less (return 1 if XL < XH)
;-------------------------------------------------------------------------------
expar_lt: 	cp	XL,XH			;SEC-TOS
		brcc	expar_eq1
expar_lt1:	ldi	XL,0x01			;result is 1
		ret

;------------------------------------------------------------------------------
; get char and change to upcase
;------------------------------------------------------------------------------
expar_gchar:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,'a'		;a
		brcs	expar_gchar_e
		cpi	tempreg1,'z'+1		;z+1
		brcc	expar_gchar_e
		subi	tempreg1,0x20		;-32
expar_gchar_e:	ret

;------------------------------------------------------------------------------
; convert hex/dec char 
;------------------------------------------------------------------------------
expar_gnumz:	rcall	bas_setupper0
expar_gnum:	cpi	tempreg1,'0'		;start
		brcs	expar_gnum_e		;error
		cpi	tempreg1,'9'+1
		brcc	expar_gnum_1
		subi	tempreg1,'0'
		ret
expar_gnum_1:	brtc	expar_gnum_e		;check only decimal
		cpi	tempreg1,'a'		;a
		brcs	expar_gnum_2
		cpi	tempreg1,'z'+1		;z+1
		brcc	expar_gnum_2
		subi	tempreg1,0x20		;-32
expar_gnum_2:	cpi	tempreg1,'A'		;a
		brcs	expar_gnum_e
		cpi	tempreg1,'Z'+1		;z+1
		brcc	expar_gnum_e
		subi	tempreg1,'A'-10
expar_gnum_e:	ret

;------------------------------------------------------------------------------
; write value
;------------------------------------------------------------------------------
expar_wval:	cpi	tempreg4,0x80		;last was op?
		brcs	expar_wval1		;no
		ldi	ereg,err_expr		;missing operator
		rjmp	expar_errend		;error handling

expar_wval1:	st	-Y,XL			;write value
		ldi	tempreg4,0x80		;last element is a value
		rjmp	expar_loop		;next char

;------------------------------------------------------------------------------
; known function keywords
;------------------------------------------------------------------------------
expar_keywords:
		.db	"CFRVADINEPKYNO",0,0

