;################################################################################
;#										#
;# avr-chipbasic8 - single chip basic computer with ATmega8			#
;# Flash memory interface							#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

;------------------------------------------------------------------------------
; the main routine
;------------------------------------------------------------------------------
file_main:	push_ptr
		push	tempreg1
		push	tempreg2
		push	tempreg3
		clt
		rcall	file_screen
		ldi	XL,0x01		;cyan on blue
		libmio_setcolor
		ldi	YL,16		;X1
		ldi	YH,10		;Y1
		ldi	ZL,45		;X2
		ldi	ZH,21		;Y2
		libmio_box
		ldi	YL,LOW(libmio_vram+6*libmio_cols+10)	;char memory
		ldi	YH,HIGH(libmio_vram+6*libmio_cols+10)
		ldi	tempreg4,0	;line counter
file_main_01:	ldi	tempreg3,12	;12 chars
		ldi	XL,0xf4		;start
		mov	XH,tempreg4
		add	XH,tempreg4
		inc	XH
file_main_02:	libi2c_read		;get char to tr2
		libmio_wms
		cpi	ereg,0
		brne	file_main_end
		sbrc	tempreg2,7
		ldi	tempreg2,'-'
		st	Y+,tempreg2
		dec	tempreg3
		brne	file_main_02
		adiw	YL,libmio_cols-12
		inc	tempreg4
		cpi	tempreg4,4
		brne	file_main_01
		rcall	file_menu
file_main_end:	set
		rcall	file_screen
		clr	ereg
file_main_e:	pop	tempreg3
		pop	tempreg2
		pop	tempreg1
		pop_ptr

;------------------------------------------------------------------------------
; menusel
;------------------------------------------------------------------------------
file_menu:	ldi	YL,LOW(libmio_vram+6*libmio_cols+9)	;char memory
		ldi	YH,HIGH(libmio_vram+6*libmio_cols+9)
		clr	tempreg4				;0 sel
		ldi	r16,0x0f				;full
file_menu_00:	st	Y,r16
file_menu_01:	libmio_waitkey
		cpi	tempreg1,0xed		;ESC
		brne	file_menu_02		;no
		ldi	tempreg4,0xff		;no sel
		rjmp	file_menu_end		
file_menu_02:	cpi	tempreg1,'l'		;load
		breq	file_menu_end		;no
		cpi	tempreg1,'s'		;save
		brne	file_menu_03		;no
		ori	tempreg4,0x08		;+8
		rjmp	file_menu_end		
file_menu_03:	cpi	tempreg1,0xe5		;down
		brne	file_menu_04		;no
		cpi	tempreg4,0x03		;last
		breq	file_menu_01		;yes
		inc	tempreg4
		st	Y,const_0
		adiw	YL,libmio_cols
		rjmp	file_menu_00
file_menu_04:	cpi	tempreg1,0xe4		;up
		brne	file_menu_01		;no
		cpi	tempreg4,0x00		;last
		breq	file_menu_01		;yes
		dec	tempreg4
		st	Y,const_0
		sbiw	YL,libmio_cols
		rjmp	file_menu_00

file_menu_end:	ret

;------------------------------------------------------------------------------
; save(T=0)/restore(T=1) screen
;------------------------------------------------------------------------------
file_screen:	ldi	XL,LOW(libmio_vram+5*libmio_cols+8)		;char memory
		ldi	XH,HIGH(libmio_vram+5*libmio_cols+8)
		ldi	ZL,LOW(libmio_vram+libmio_cols*libmio_rows)	;temp memory
		ldi	ZH,HIGH(libmio_vram+libmio_cols*libmio_rows)
		ldi	tempreg1,6		;rows
file_screen_1:	ldi	tempreg2,15		;cols
file_screen_2:	brts	file_screen_3		;branch if restore
		ld	tempreg3,X		;get char
		st	Z+,tempreg3		;save
		ldi	tempreg3,0x20		;space
		st	X+,tempreg3		;replace char with space
		rjmp	file_screen_4		;goto loop

file_screen_3:	ld	tempreg3,Z+		;get char
		st	X+,tempreg3		;restore
file_screen_4:	dec	tempreg2		;col counter
		brne	file_screen_2		;column loop
		adiw	XL,15			;next line
		dec	tempreg1		;row counter
		brne	file_screen_1
		brts	file_screen_e		;goto end
file_screen_e:	ret
