;################################################################################
;#										#
;# avr-chipbasic0 - single chip basic computer with ATmega8			#
;# interpreter engine								#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.equ	err_end		=	0x00			;normal end
.equ	err_break	=	0x01			;break key
.equ	err_serr	=	0x02			;no keyword (syntax error)
.equ	err_expr	=	0x03
.equ	err_for		=	0x04
.equ	err_next	=	0x04
.equ	err_gosub	=	0x04
.equ	err_return	=	0x04
.equ	err_array	=	0x05
.equ	bas_stackp	=	bas_ram+1
;Code im ROM

runprog:	libmio_setnormal
		mov	bas_line,const_0	;set to start
		mov	bas_statement,const_0
		sts	libmio_ram+libmio_kflags,const_0	;clear break flag
		clr	ereg			;clear error
;clear up variables
bas_clear:	ldi	YL,LOW(varspace)	;variables area
		ldi	YH,HIGH(varspace)
		ldi	XL,26
bas_clear_1:	st	Y+,const_0
		dec	XL
		brne	bas_clear_1
		sts	bas_stackp,const_0	;clear stack


interloop:	ldi	ZL,20			;max lines
		cp	bas_line,ZL		;check line number
		brcs	interloop_01		;is OK
		ldi	ereg,err_end		;end code
		ret				;end
		
interloop_01:	ldi	ZH,HIGH(bas_programs*2)	;program base
		ldi	ZL,25			;chars per line
		mul	ZL,bas_line		;calc line offset
		clr	ZL
		add	ZL,r0			;
		adc	ZH,r1
		mov	ctrl,ZL
		subi	ctrl,-26		;set to address of next line
		clr	r16			;we are at statement 1

;Z points to the start of line, now skip statements

interloop_02:	cp	r16,bas_statement	;we are at the right statement?
		breq	interloop_08		;yes

interloop_03:	lpm	tempreg1,Z+		;get char
		cp	ZL,ctrl			;EOL
		brne	interloop_05		;no
interloop_04:	mov	bas_statement,const_0	;statement=0
		inc	bas_line		;next line
		rjmp	interloop

interloop_05:	cpi	tempreg1,0x27		;comment
		breq	interloop_04		;->next line

		cpi	tempreg1,'"'		;start of string
		brne	interloop_07		;no

interloop_06:	lpm	tempreg1,Z+		;get char
		cp	ZL,ctrl			;EOL
		breq	interloop_04		;yes
		cpi	tempreg1,'"'		;end of string
		brne	interloop_06		;no
		rjmp	interloop_03		;check next char
		
interloop_07:	cpi	tempreg1,':'		;statement separator :
		brne	interloop_03		;no
		inc	r16			;recent statement+1
		rjmp	interloop_02		;statement loop
		
;Z points to the right statement
interloop_08:	lds	tempreg1,libmio_ram+libmio_kflags
		sbrs	tempreg1,0
		rjmp	interloop_08c

interloop_08a:	ldi	ereg,err_break
		sts	libmio_ram+libmio_kflags,const_0
		ret

interloop_08c:	libmio_screenshot

interloop_09:	lpm	tempreg1,Z+		;get char
		cp	ZL,ctrl			;EOL
		breq	interloop_04		;yes

		cpi	tempreg1,0x27		;comment
		breq	interloop_04

		cpi	tempreg1,0x20		;space
		breq	interloop_08		;ignore

		cpi	tempreg1,'?'		;print
		brne	interloop_10		;no
		rjmp	bas_print_00

interloop_nk:	ldi	ereg,err_serr		;no keyword
interloop_r1:	ret				;return

interloop_10:	rcall	bas_cchar		;set to uppercase
		brts	interloop_nk		;error
		mov	tempreg2,tempreg1	;save char
		lpm	tempreg1,Z		;next char
		cpi	tempreg1,'='		;=set variable
		brne	interloop_12		;no

;variable setting
		adiw	ZL,1			;point to next char
		mov	r15,tempreg2		;save variable number
		rcall	expar			;call expression parser
		mov	tempreg2,r15		;restore variable number
		ldi	YL,LOW(varspace-'A')	;variable base
		ldi	YH,HIGH(varspace-'A')
		add	YL,tempreg2		;add offset
		adc	YH,const_0
		st	Y+,XL			;store value
		rjmp	bas_cont

;copy to RAM-buffer
interloop_12:	ldi	YL,LOW(bas_buf)		;basic buffer
		ldi	YH,HIGH(bas_buf)	
		rcall	bas_setupper
		st	Y+,tempreg2		;store first char
		st	Y+,tempreg1		;store second char
		ldi	tempreg2,'.'		;filler
		st	Y,tempreg2		;char 3
		std	Y+1,tempreg2		;char 4
		ldi	tempreg2,0x02		;max 4 chars
		
interloop_14:	adiw	ZL,1			;increment pointer
		lpm	tempreg1,Z		;get char
		rcall	bas_cchar
		brts	interloop_16		;no a-z
		st	Y+,tempreg1		;store to buffer
		dec	tempreg2		;char counter
		brne	interloop_14		;loop

;Z points to first char after Keyword
interloop_16:	movw	tempreg7,ZL		;save pointer
		ldi	ZH,HIGH(bas_keywords*2)	;keyword table
		ldi	ZL,LOW(bas_keywords*2)	;
interloop_18:	ldi	YL,LOW(bas_buf)		;basic buffer
		ldi	YH,HIGH(bas_buf)
		lpm	XL,Z+			;low address
		lpm	XH,Z+			;high address
		ldi	tempreg2,0x04		;number of chars
interloop_20:	lpm	tempreg1,Z+		;get char
		cpi	tempreg1,'.'		;dont care
		breq	interloop_21		;yes
		ld	tempreg3,Y+		;get RAM char
		cp	tempreg3,tempreg1	;compare
		brne	interloop_24		;not equal
interloop_21:	dec	tempreg2		;char counter
		brne	interloop_20		;
;keyword ist found				
interloop_22:	movw	ZL,XL			;copy jump address
		ijmp				;jump to routine

interloop_24:	dec	tempreg2		;remaining chars -1
		add	ZL,tempreg2		;add remaining chars
		adc	ZH,const_0
		rjmp	interloop_18

;------------------------------------------------------------------------------
;keyword is PRINT 
;------------------------------------------------------------------------------
bas_print:	movw	ZL,tempreg7		;restore pointer
bas_print_00:	sts	libmio_ram+libmio_cflag,const_0	;clear collision flag
bas_print_01:	mov	tempreg5,const_1	;enable newline
bas_print_02:	lpm	XH,Z+
		cp	ZL,ctrl			;end of line
		breq	bas_print_end
		cpi	XH,0x20			;leading space
		breq	bas_print_02
		cpi	XH,':'			;end of statement
		breq	bas_print_end
		rjmp	bas_print_sep
bas_print_end:	sbiw	ZL,1			;points to this char
		sbrc	tempreg5,0			;newline?
		libmio_newline
bas_print_end1:	rjmp	bas_cont		;return to main loop
	
;-------------- separator ----------------------------------------------
bas_print_sep:	cpi	XH,0x3b			;";"
		breq	bas_print_tab3

;-------------- tabulator ----------------------------------------------
bas_print_tab:	cpi	XH,','			;tab (,)
		brne	bas_print_05

bas_print_tab1:	libmio_outspace
		mov	XH,libmio_cur_x
		andi	XH,0x03			;tab size is 4
		brne	bas_print_tab1
		
bas_print_tab3:	clr	tempreg5		;no newline
	    	rjmp	bas_print_02		;print loop1

;-------------- enable newline if no separator --------------------------
bas_print_05:	mov	tempreg5,const_1	;enable newline

;------------- set position --------------------------------------------------------
bas_print_at:	cpi	XH,'@'			;position set?
		brne	bas_print_str		;no
		rcall	expar			;get Y
		mov	XH,XL			;store Y
		adiw	ZL,1			;skip separator
		rcall	expar			;get X
		libmio_gotoxy			;set position
		rjmp	bas_print_01		;print loop

;-------------- a string ------------------------------------------------
bas_print_str:	cpi	XH,'"'			;is it a string?
		brne	bas_print_char		;no
bas_print_str1:	lpm	tempreg1,Z+		;get string char
		cp	ZL,ctrl
		breq	bas_print_end		;yes
		cpi	tempreg1,'"'		;end of string
		breq	bas_print_01		;yes
		libmio_outchar			;output char
		rjmp	bas_print_str1		;string loop

;------------- a single char --------------------------------------------
bas_print_char:	cpi	XH,'%'			;char %
		brne	bas_print_equ
		rcall	expar
		mov	tempreg1,XL
		libmio_outchar
		rjmp	bas_print_01

;------------- an equation --------------------------------------------
bas_print_equ:	sbiw	ZL,1			;set pointer to char before
		rcall	expar			;mathematics
		libmio_outdez
		rjmp	bas_print_01

;------------------------------------------------------------------------------
;keyword is COLOR
;------------------------------------------------------------------------------
bas_color:	rcall	expar0			;get foregound
		andi	XL,0x09			;mask bits
		libmio_setcolor
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is SCROLL
;------------------------------------------------------------------------------
bas_scroll:	rcall	expar0			;get foregound
		mov	tempreg1,XL
		libmio_scroll
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is GET
;------------------------------------------------------------------------------
bas_get:	libmio_getser
		rcall	bas_chkvar
		st	X,tempreg2
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is PUT
;------------------------------------------------------------------------------
bas_put:	rcall	expar0			;get foregound
		mov	tempreg2,XL
		libmio_putser
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is TONE
;------------------------------------------------------------------------------
bas_tone:	rcall	expar0			;get foregound
		sts	libmio_ram+libmio_tone,XL
		ldi	XL,0xff
		sts	libmio_ram+libmio_envval,XL
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is PLOT
;------------------------------------------------------------------------------
bas_plot:	rcall	expar0			;get Y
		mov	XH,XL
		adiw	ZL,1			;skip separator
		rcall	expar			;get X
		libmio_plot
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is BOX
;------------------------------------------------------------------------------
bas_box:	rcall	bas_g4
		cpi	ereg,0
		brne	bas_box_e
		libmio_box
		movw	ZL,tempreg7		;restore pointer
		rjmp	bas_cont		
bas_box_e:	ret

;------------------------------------------------------------------------------
;keyword is FBOX
;------------------------------------------------------------------------------
bas_fbox:	rcall	bas_g4
		cpi	ereg,0
		brne	bas_box_e
		libmio_fbox
		movw	ZL,tempreg7		;restore pointer
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is DRAW
;------------------------------------------------------------------------------
bas_draw:	rcall	bas_g4
		cpi	ereg,0
		brne	bas_box_e
		libmio_draw
		movw	ZL,tempreg7		;restore pointer
		rjmp	bas_cont

;------------------------------------------------------------------------------
;keyword is GOTO
;------------------------------------------------------------------------------
bas_goto:	rcall	expar0			;get parameter
		dec	XL
		mov	bas_line,XL
		mov	bas_statement,const_0
		rjmp	interloop

;------------------------------------------------------------------------------
;keyword is CLS
;------------------------------------------------------------------------------
bas_cls:	movw	ZL,tempreg7		;restore pointer
		libmio_clrscr
		rjmp	bas_cont

;-----------------------------------------------------------------------
; keyword is IF
;-----------------------------------------------------------------------
bas_if:		rcall	expar0
		cpi	XL,0x00			;false
		brne	bas_if_1		;all other is true
		inc	bas_line		;next line
		mov	bas_statement,const_0
		rjmp	interloop		;next line
bas_if_1:	adiw	ZL,1			;ignore next
		rjmp	bas_cont_01

;-----------------------------------------------------------------------
; keyword is FOR
;-----------------------------------------------------------------------
bas_for:	rcall	bas_chkvar
		lpm	tempreg1,Z+		;space
		cpi	tempreg1,'='		;=
		brne	bas_for_err		;syntax error
		call	expar			;call parser
bas_for_1:	ldi	YH,high(varspace)
		ldi	YL,low(varspace)
		add	YL,tempreg7
		adc	YH,const_0
		st	Y+,XL			;set variable to initial value
		rcall	bas_ispace
		rcall	bas_setupper
		cpi	tempreg1,'T'		;T
		brne	bas_for_err
		rcall	bas_setupper0
		cpi	tempreg1,'O'		;O
		brne	bas_for_err
		call	expar			;target value
		mov	tempreg5,XL		;save target value

bas_for_2:	rcall	bas_gstack		;stackpointer
		cpi	XH,bas_stackdepth-1	;maximum
		brcs	bas_for_3

bas_for_err:	ldi	ereg,err_for		;too many for
		ret

bas_for_3:	rcall	bas_cstack		;calc stack address
		rcall	bas_istack		;set new stackpointer
		mov	XH,bas_statement	;statement number
		inc	XH			;next must skip this
		st	Y+,XH			;statement number
		st	Y+,bas_line		;line number
		st	Y+,tempreg7		;variable offset
		st	Y+,tempreg5		;target value
		rjmp	bas_cont		;go on...

;-----------------------------------------------------------------------
; keyword is NEXT
;-----------------------------------------------------------------------
bas_next:	movw	ZL,tempreg7		;restore pointer
		rcall	bas_gstack		;stackpointer
		cpi	XH,0
		brne	bas_next_02
bas_next_01:	ldi	ereg,err_next		;next without for
		ret

bas_next_02:	dec	XH
		rcall	bas_cstack
		ld	XH,Y			;statement
		sbrc	XH,7			;check if TOS is for
		rjmp	bas_next_01		;no

		ldd	tempreg4,Y+2		;variable
		ldi	XH,high(varspace)	
		ldi	XL,low(varspace)
		add	XL,tempreg4
		adc	XH,const_0
		ld	r16,X+			;variable value
		inc	r16			;+1
		st	-X,r16			;store back
		ldd	XL,Y+3			;target
		inc	XL			;+1
		cp	r16,XL			;done?
		brne	bas_next_03
		rcall	bas_gstack		;get stackpointer
		rcall	bas_dstack		;decrement and save back
		rjmp	bas_cont		;next statement

bas_next_03:	ld	bas_statement,Y		;set values for looping
		ldd	bas_line,Y+1
		rjmp	interloop		;calc new

;-----------------------------------------------------------------------
; keyword is GOSUB
;-----------------------------------------------------------------------
bas_gosub:	rcall	expar0			;get parameter
		dec	XL

bas_gosub_02:	rcall	bas_gstack		;stackpointer
		cpi	XH,bas_stackdepth-1	;maximum-1
		brcs	bas_gosub_03
		ldi	ereg,err_gosub		;too many gosub
		ret				;goto main interpreter loop

bas_gosub_03:	push	XL			;this is the new linenumber
		rcall	bas_cstack
		adc	YH,r1
		rcall	bas_istack
		mov	XH,bas_statement	;statement
		inc	XH			;must return to next statement
		ori	XH,0x80			;mark as gosub
		st	Y+,XH			;return statement number
		mov	XH,r12			;our line number
		st	Y+,bas_line
		mov	bas_statement,const_0	;statement=0
		pop	bas_line		;new linenumber
		rjmp	interloop		;end of working on this line

;-----------------------------------------------------------------------
; keyword is RETURN
;-----------------------------------------------------------------------
bas_return:	movw	ZL,tempreg7		;restore pointer
		rcall	bas_gstack		;stackpointer
		cpi	XH,0
		brne	bas_return_02
bas_return_01:	ldi	ereg,err_return		;return without call
		ret

bas_return_02:	dec	XH
		rcall	bas_cstack
		rcall	bas_pstack
		ld	XH,Y+	
		sbrs	XH,7			;check if TOS is call
		rjmp	bas_return_01		;no
		andi	XH,0x7f
		mov	bas_statement,XH	;statement
		ld	bas_line,Y+		;linenumber
		rjmp	interloop

;-----------------------------------------------------------------------
; keyword is EPOKE
;-----------------------------------------------------------------------
bas_epoke:	rcall	bas_expar0		;get address
		adiw	ZL,1
		mov	YL,XL			;address
		rcall	bas_expar		;get data
		clr	YH
bas_epoke0:	mov	tempreg1,XL
		libeep_write
bas_epoke1:	rjmp	bas_cont

;-----------------------------------------------------------------------
; keyword is CFG
;-----------------------------------------------------------------------
bas_cfg:	rcall	bas_expar0		;get data
		sts	libmio_ram+libmio_sysconf,XL
		ldi	YL,0xff
		ldi	YH,0x01
		rjmp	bas_epoke0

;-----------------------------------------------------------------------
; keyword is DIR
;-----------------------------------------------------------------------
bas_dir:	rcall	bas_expar0
		ori	XL,0xf0			;mask bits
		out	ioport_ddr,XL
bas_dir_e:	rjmp	bas_cont

;-----------------------------------------------------------------------
; keyword is SYNC
;-----------------------------------------------------------------------
bas_sync:	rcall	bas_expar0
bas_sync_1:	cpi	XL,0x00
		breq	bas_dir_e
		libmio_sync
		dec	XL
		rjmp	bas_sync_1

;-----------------------------------------------------------------------
; keyword is OUT (Y=pin XL=val)
;-----------------------------------------------------------------------
bas_out:	rcall	bas_expar0
		ori	XL,0xf0			;mask bits
		out	ioport_out,XL
		rjmp	bas_cont

;-----------------------------------------------------------------------
; keyword is VID 
;-----------------------------------------------------------------------
bas_vid:	rcall	bas_expar0
		com	XL
		sts	libmio_ram+libmio_fastmode,XL
		rjmp	bas_cont

;-----------------------------------------------------------------------
; keyword is INV
;-----------------------------------------------------------------------
bas_inv:	rcall	bas_chkvar
		neg	r16
		rjmp	bas_setvar

;-----------------------------------------------------------------------
; keyword is INPUT
;-----------------------------------------------------------------------
bas_input:	rcall	bas_chkvar
		libmio_calc			;get video ptr
		push	YL			;save ptr
		push	YH
		ldi	tempreg1,0x04		;4 chars
		libmio_input
		libmio_newline
		pop	YH			;restore ptr
		pop	YL
		cpi	tempreg1,0xff		;break with ESC
		brne	bas_input_01		;no
		rjmp	bas_cont		;goto main loop
		
bas_input_err:	rjmp	interloop_nk

;calc value
bas_input_01:	clr	r16			;value
		ldi	tempreg2,4		;number of chars
bas_input_02:	ld	tempreg1,Y+		;get next char
		subi	tempreg1,'0'		;'0'
		cpi	tempreg1,0x0a
		brcc	bas_input_03
		ldi	XH,10
		mul	XH,r16			;value*10
		mov	r16,r0
		add	r16,tempreg1
		dec	tempreg2
		brne	bas_input_02
bas_input_03:	rjmp	bas_setvar		;set variable and continue

;-----------------------------------------------------------------------
; keyword is LIM
;-----------------------------------------------------------------------
bas_limit:	rcall	bas_chkvar		;set var
		adiw	ZL,1			;skip separator
		rcall	expar			;get min
		cp	XL,r16
		brcs	bas_limit_1
		mov	r16,XL
bas_limit_1:	adiw	ZL,1			;skip separator
		rcall	expar			;get max
		cp	XL,r16
		brcc	bas_limit_2
		mov	r16,XL
bas_limit_2:	rjmp	bas_setvar

;-----------------------------------------------------------------------
; check for valid variable and set tempreg7 to offset+read r16
;-----------------------------------------------------------------------
bas_chkvar:	movw	ZL,tempreg7		;restore pointer
		rcall	bas_ispace		;space
		rcall	bas_cchar
		brts	bas_chkvar_err
		subi	tempreg1,'A'
		mov	tempreg7,tempreg1	;store variable number
		ldi	XL,LOW(varspace)
		ldi	XH,HIGH(varspace)
		add	XL,tempreg7
		adc	XH,const_0
		ld	r16,X
		ret
bas_chkvar_err:	pop	tempreg7
		pop	tempreg7
		rjmp	interloop_nk

;-----------------------------------------------------------------------
; set variable tempreg7 to r16 and continue
;-----------------------------------------------------------------------
bas_setvar:	ldi	XL,LOW(varspace)
		ldi	XH,HIGH(varspace)
		add	XL,tempreg7
		adc	XH,const_0
		st	X,r16			;store value
		rjmp	bas_cont		;goto main loop

;-----------------------------------------------------------------------
; read until char is not space
;-----------------------------------------------------------------------
bas_ispace:	lpm	tempreg1,Z+
bas_ispc:	cpi	tempreg1,0x20		;space
		breq	bas_ispace
		ret
	
;-----------------------------------------------------------------------
; call exparser and correct pointer
;-----------------------------------------------------------------------
bas_expar0:	movw	ZL,tempreg7
bas_expar:	rcall	bas_ispace
		sbiw	ZL,1
		rjmp	expar

;-----------------------------------------------------------------------
; continue
;-----------------------------------------------------------------------

bas_cont:	clr	ereg			;clear error
		lpm	tempreg1,Z+		;get char
		cp	ZL,ctrl			;end
		breq	bas_cont_02
		cpi	tempreg1,':'		;next statement
		breq	bas_cont_01
		rjmp	bas_cont	
		
bas_cont_01:	rjmp	interloop_08
bas_cont_02:	inc	bas_line
		mov	bas_statement,const_0
		rjmp	interloop

;------------------------------------------------------------------------------
;get 4 values
;------------------------------------------------------------------------------
bas_g4:		movw	ZL,tempreg7		;restore pointer
		ldi	YL,LOW(bas_buf)
		ldi	YH,HIGH(bas_buf)
		ldi	tempreg4,0x04
bas_g4_1:	push	tempreg4
		rcall	expar			;get Y1
		st	Y+,XL
		adiw	ZL,1			;skip separator
		pop	tempreg4
		dec	tempreg4
		brne	bas_g4_1
		sbiw	YL,4
		sbiw	ZL,1
		movw	tempreg7,ZL		;save pointer
		mov	ZL,XL			;X2
		ld	ZH,Y+
		ld	XL,Y+
		ld	XH,Y+
		movw	YL,XL
		ret

;------------------------------------------------------------------------------
;calc stack pos (XH)
;------------------------------------------------------------------------------
bas_cstack:	ldi	YL,LOW(bas_stacks)	;stack root
		ldi	YH,HIGH(bas_stacks)
		ldi	XL,4			;bytes/record
		mul	XL,XH
		add	YL,r0
		adc	YH,r1
		ret

;------------------------------------------------------------------------------
;get stack ptr
;------------------------------------------------------------------------------
bas_gstack:	lds	XH,bas_stackp
		ret

;------------------------------------------------------------------------------
;put stack ptr
;------------------------------------------------------------------------------
bas_dstack:	subi	XH,2
bas_istack:	inc	XH
bas_pstack:	sts	bas_stackp,XH
		ret

;------------------------------------------------------------------------------
;set tempreg1 to uppercase
;------------------------------------------------------------------------------
bas_setupper0:	lpm	tempreg1,Z+
bas_setupper:	cpi	tempreg1,'a'		;first lower char
		brcs	bas_setupper1		;below
		subi	tempreg1,0x20		;-32
bas_setupper1:	ret

;------------------------------------------------------------------------------
;set tempreg1 to uppercase and check for a-z
;------------------------------------------------------------------------------
bas_cchar:	rcall	bas_setupper
		set				;fail
		cpi	tempreg1,'A'
		brcs	bas_cchar_1
		cpi	tempreg1,'Z'+1
		brcc	bas_cchar_1
		clt				;pass
bas_cchar_1:	ret

;------------------------------------------------------------------------------
; the keyword table
;------------------------------------------------------------------------------
bas_keywords:
;plot
		.db	LOW(bas_plot),HIGH(bas_plot)
		.db 	"PL.."

;box
		.db	LOW(bas_box),HIGH(bas_box)
		.db 	"BX.."

;fbox
		.db	LOW(bas_fbox),HIGH(bas_fbox)
		.db 	"FB.."	

;draw
		.db	LOW(bas_draw),HIGH(bas_draw)
		.db 	"DR.."

;if
		.db	LOW(bas_if),HIGH(bas_if)
		.db 	"IF.."

;limit
		.db	LOW(bas_limit),HIGH(bas_limit)
		.db 	"LIM."

;for
		.db	LOW(bas_for),HIGH(bas_for)
		.db 	"FOR."

;next
		.db	LOW(bas_next),HIGH(bas_next)
		.db 	"NXT."

;sub
		.db	LOW(bas_gosub),HIGH(bas_gosub)
		.db 	"SUB."

;go
		.db	LOW(bas_goto),HIGH(bas_goto)
		.db 	"GO.."

;ret
		.db	LOW(bas_return),HIGH(bas_return)
		.db 	"RET."

;epoke
		.db	LOW(bas_epoke),HIGH(bas_epoke)
		.db 	"POK."

;dir
		.db	LOW(bas_dir),HIGH(bas_dir)
		.db 	"DIR."

;out
		.db	LOW(bas_out),HIGH(bas_out)
		.db 	"OUT."

;color
		.db	LOW(bas_color),HIGH(bas_color)
		.db 	"CO.."

;cls
		.db	LOW(bas_cls),HIGH(bas_cls)
		.db 	"CLS."

;input
		.db	LOW(bas_input),HIGH(bas_input)
		.db 	"INP."

;sync
		.db	LOW(bas_sync),HIGH(bas_sync)
		.db 	"SY.."

;inv
		.db	LOW(bas_inv),HIGH(bas_inv)
		.db 	"INV."		

;no
		.db	LOW(bas_tone),HIGH(bas_tone)
		.db 	"NO.."		

;put
		.db	LOW(bas_put),HIGH(bas_put)
		.db 	"PUT."		

;get
		.db	LOW(bas_get),HIGH(bas_get)
		.db 	"GET."		

;scroll
		.db	LOW(bas_scroll),HIGH(bas_scroll)
		.db 	"SCR."		

;vid
		.db	LOW(bas_vid),HIGH(bas_vid)
		.db 	"VID."		

;cfg
		.db	LOW(bas_cfg),HIGH(bas_cfg)
		.db 	"CFG."		

;default=error
		.db	LOW(interloop_nk),HIGH(interloop_nk)
		.db	"...."			;error (matches ever)
