;################################################################################
;#										#
;# avr-chipbasic2 - single chip basic computer with ATmega644			#
;# EEPROM/Flash memory interface						#
;# copyright (c) 2006-2009 Joerg Wolfram (joerg@jcwolfram.de)			#
;#										#
;#										#
;# This program is free software; you can redistribute it and/or		#
;# modify it under the terms of the GNU General Public License			#
;# as published by the Free Software Foundation; either version 3		#
;# of the License, or (at your option) any later version.			#
;#										#
;# This program is distributed in the hope that it will be useful,		#
;# but WITHOUT ANY WARRANTY; without even the implied warranty of		#
;# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU		#
;# General Public License for more details.					#
;#										#
;# You should have received a copy of the GNU General Public			#
;# License along with this library; if not, write to the			#
;# Free Software Foundation, Inc., 59 Temple Place - Suite 330,			#
;# Boston, MA 02111-1307, USA.							#
;#										#
;################################################################################

.macro spmcr_in
.if ctype==8
		in	tempreg1,SPMCR
.endif
.if ctype==88
		in	tempreg1,SPMCSR
.endif
.endmacro

.macro spmcr_out
.if ctype==8
		out	SPMCR,XH
.endif
.if ctype==88
		out	SPMCSR,XH
.endif
.endmacro

;------------------------------------------------------------------------------
; subroutines
;------------------------------------------------------------------------------
mem_wrpage:	sbiw	ZL,2
		ldi	XH,0x05			;page write
		rcall	mem_spm
		rcall	mem_waitrww
		adiw	ZL,2
		ret

mem_erase:	ldi	XH,0x03			;page erase
		rcall	mem_spm
		rjmp	mem_waitrww

mem_spm:	spmcr_in
		sbrc	tempreg1,0
		rjmp	mem_spm
mem_spm1:	spmcr_out
		spm
mem_spm2:	spmcr_in
		sbrc	tempreg1,0
		rjmp	mem_spm2
		ret

mem_waitrww:	spmcr_in
		sbrs	tempreg1,6
		ret
		ldi	XH,0x11
		rcall	mem_spm
		rjmp	mem_waitrww

mem_putbyte:	sbrc	r16,0			;skip if even cell
		rjmp	mem_putbyte_1		;jump to odd routine
		mov	r0,tempreg1		;low byte
		inc	r16			;increment counter
		ret

mem_putbyte_1:	mov	r1,tempreg1		;high byte
		ldi	XH,0x01			;write to buffer
		rcall	mem_spm
		adiw	ZL,2			;ptr+2
		inc	r16			;increment counter
		mov	r17,r16			;copy counter
		andi	r17,0x3f		;63 bytes
		brne	mem_putbyte_2
		rcall	mem_wrpage
mem_putbyte_2:	ret

;------------------------------------------------------------------------------
; write program to flash
;------------------------------------------------------------------------------
mem_save:	cli				;diable interrupt
		ldi	ZL,LOW(bas_programs*2)	;base
		ldi	ZH,HIGH(bas_programs*2)
		ldi	tempreg2,8		;8x64 Bytes
mem_progclr:	rcall	mem_erase		;erase page 1
		ldi	tempreg1,0x40
		add	ZL,tempreg1		;next page
		adc	ZH,const_0
		dec	tempreg2
		brne	mem_progclr
		subi	ZH,2
		ldi	YL,LOW(libmio_vram+2*libmio_cols+4)
		ldi	YH,HIGH(libmio_vram+2*libmio_cols+4)
		ldi	tempreg2,20
		clr	r16			;reset counter
mem_save_01:	ldi	tempreg3,25		;chars per line
mem_save_02:	ld	tempreg1,Y+
		rcall	mem_putbyte
		dec	tempreg3
		brne	mem_save_02
		adiw	YL,5
		dec	tempreg2
		brne	mem_save_01
		ldi	YL,LOW(libmio_vram+11)
		ldi	YH,HIGH(libmio_vram+11)
		ldi	tempreg2,12
mem_save_03:	ld	tempreg1,Y+
		rcall	mem_putbyte
		dec	tempreg2
		brne	mem_save_03
		sei
		ret

;------------------------------------------------------------------------------
;get text from flash (T=1) or verify (T=0)
;------------------------------------------------------------------------------
mem_load:	ldi	ZL,LOW(bas_programs*2)
		ldi	ZH,HIGH(bas_programs*2)
		ldi	YL,LOW(libmio_vram+(libmio_cols*2)+4)
		ldi	YH,HIGH(libmio_vram+(libmio_cols*2)+4)
		ldi	tempreg1,20		;number of lines
mem_load_1:	ldi	tempreg2,25		;number of chars
mem_load_2:	lpm	XL,Z+			;get ROM char
		brts	mem_load_3		;skip if load mode
		ld	XH,Y			;get RAM char
		cp	XL,XH			;compare
		brne	mem_load_ve

mem_load_3:	st	Y+,XL			;store char
		dec	tempreg2
		brne	mem_load_2

mem_load_4:	adiw	YL,5			;skip line number
		dec	tempreg1
		brne	mem_load_1
mem_load_5:	ldi	YL,LOW(libmio_vram+11)
		ldi	YH,HIGH(libmio_vram+11)
		ldi	ZL,LOW(bas_programs*2+500)
		ldi	ZH,HIGH(bas_programs*2+500)
		ldi	tempreg1,12
mem_load_6:	lpm	XL,Z+
		st	Y+,XL
		dec	tempreg1
		brne	mem_load_6
		ret

mem_load_ve:	set				;verify error
		ret

;------------------------------------------------------------------------------
;charout (X+) both nibbles +0x20
;------------------------------------------------------------------------------
mem_cout:	ld	tempreg2,X+
mem_cout1:	push	tempreg2
		andi	tempreg2,0xf0
		swap	tempreg2
		subi	tempreg2,0xbf		;+"A"
		libmio_putser
		pop	tempreg2
		andi	tempreg2,0x0f
		subi	tempreg2,0xbf		;+"A"
		libmio_putser
		ret
